// HDTVtoMPEG2Dlg.cpp : implementation file
//
// Changelog:
//
// 1.07 - 2-23-2002 - GR: AccessDTV file support.
//						Avoid duplicate input files.
//						Support for progress bar of total conversion.
//
#include "stdafx.h"
#include "HDTVtoMPEG2.h"
#include "HDTVtoMPEG2Dlg.h"
#include "process.h"
#include "mpeg2stuff.h"
#include "math.h"
#include "Shlwapi.h"
#include "ConfigureDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

static char temp[256];
static char inipath[256];

payload_buff Stream_Header; // saved copy of stream_header
PatPmt_Packets PatPmtPackets; // saved copies of PAT and PMT packets

char *filetypeNames[] =
{
	"Transport Stream",
	"HiDTV",
	"Mpeg 2",
};

char *default_extensions[] =
{
	".0000.ts",
	".tls",
	".0000.mpg",
};


void makeTimeStr(char *buf, float sec)
{
	int h = int(sec/3600.0f);
	int m = int(sec/60.0f) % 60;
	int s = int(sec) % 60;

	if (h)
		sprintf(buf, "%dh%02dm%02ds", h, m, s);
	else
		sprintf(buf, "%dm%02ds", m, s);
}

void makeTimeStr2(char *buf, float sec)
{
	int h = int(sec/3600.0f);
	int m = int(sec/60.0f) % 60;
	int s = int(sec) % 60;
	int f = int(fmod(sec, 1.0f)*100);

	if (h)
		sprintf(buf, "%dh%02dm%02d.%02ds", h, m, s, f);
	else
		sprintf(buf, "%dm%02d.%02ds", m, s, f);
}

void makeSpeedStr(char *buf, float spd, char b)
{
	if (spd < 1.0e3f)
		sprintf(buf, "%.2f %c/s", spd, b);
	else if (spd < 1.0e6f)
		sprintf(buf, "%.2f K%c/s", spd * 1.0e-3f, b);
	else if (spd < 1.0e9f)
		sprintf(buf, "%.2f M%c/s", spd * 1.0e-6f, b);
	else if (spd < 1.0e12f)
		sprintf(buf, "%.2f G%c/s", spd * 1.0e-9f, b);
	else
		sprintf(buf, "%.2f T%c/s", spd * 1.0e-12f, b);
}

void makeSizeStr(char *buf, __int64 size)
{
	float f;
	char *format;
	char *s;
	if (size < 1024)
	{
		f = (float)size;
		format = "%.0f %s";
		s = "B";
	}
	else
	{
		if (size < (__int64)1024*(__int64)1024)
		{
			f = (float)size/(__int64)1024;
			s = "KB";
		}
		else if (size < (__int64)1024*(__int64)1024*(__int64)1024)
		{
			f = (float)size/((__int64)1024*(__int64)1024);
			s = "MB";
		}
		else if (size < (__int64)1024*(__int64)1024*(__int64)1024*(__int64)1024)
		{
			f = (float)size/((__int64)1024*(__int64)1024*(__int64)1024);
			s = "GB";
		}
		else
		{
			f = (float)size/((__int64)1024*(__int64)1024*(__int64)1024*(__int64)1024);
			s = "TB";
		}
		if (f < 10.0f)
			format = "%.2f %s";
		else if (f < 100.0f)
			format = "%.1f %s";
		else
			format = "%.0f %s";
	}
	sprintf(buf, format, f, s);
}


/////////////////////////////////////////////////////////////////////////////
// CAboutDlg dialog used for App About

class CAboutDlg : public CDialog
{
public:
	CAboutDlg();

// Dialog Data
	//{{AFX_DATA(CAboutDlg)
	enum { IDD = IDD_ABOUTBOX };
	//}}AFX_DATA

	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CAboutDlg)
	protected:
	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
	//}}AFX_VIRTUAL

// Implementation
protected:
	//{{AFX_MSG(CAboutDlg)
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

CAboutDlg::CAboutDlg() : CDialog(CAboutDlg::IDD)
{
	//{{AFX_DATA_INIT(CAboutDlg)
	//}}AFX_DATA_INIT
}

void CAboutDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CAboutDlg)
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CAboutDlg, CDialog)
	//{{AFX_MSG_MAP(CAboutDlg)
		// No message handlers
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

int get_hex(const char* str)
{
	if (strncmp(str, "0x", 2) != 0)
		return atol(str);

	int v = 0;	
	const char *p = str + 2;
	while (*p && *p != ' ')
	{
		if (*p >= '0' && *p <= '9')
			v = (v << 4) | (*p - '0');
		else if (*p >= 'a' && *p <= 'f')
			v = (v << 4) | ((*p - 'a') + 10);
		else if (*p >= 'A' && *p <= 'F')
			v = (v << 4) | ((*p - 'A') + 10);
		p++;
	}

	return v;
}

/////////////////////////////////////////////////////////////////////////////
// CHDTVtoMPEG2Dlg dialog

CHDTVtoMPEG2Dlg::CHDTVtoMPEG2Dlg(CWnd* pParent /*=NULL*/)
	: CDialog(CHDTVtoMPEG2Dlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CHDTVtoMPEG2Dlg)
	m_AudioPIDStr = _T("");
	m_VideoPIDStr = _T("");
	m_MaxFileSize = 0;
	//}}AFX_DATA_INIT
	// Note that LoadIcon does not require a subsequent DestroyIcon in Win32

	GetModuleFileName(NULL, inipath, 256);
	char *p = strrchr(inipath, '\\');
	if (!p)
		p = inipath;
	else
		p++;
	strcpy(p, "HDTVtoMPEG2.ini");

	m_NumChannels = 0;
	memset(m_Channels, 0, sizeof(ATSC_CHANNEL_INFO) * MAXCHANNELS);

	GetPrivateProfileString("Settings", "VideoStream", "0x11", temp, 255, inipath);
	m_VideoPIDStr = temp;
	m_VideoPID = get_hex(temp);
	GetPrivateProfileString("Settings", "AudioStream", "0x14", temp, 255, inipath);
	m_AudioPIDStr = temp;
	m_AudioPID = get_hex(temp);

	GetPrivateProfileString("Settings", "MaxFileSize", "1024", temp, 255, inipath);
	m_MaxFileSize = atol(temp);
    
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);

	m_bCancel = false;

	m_OutputFileName[0] = '\0';

	m_ProgressDlg = NULL;

	m_CurrentTimeText[0] = '\0';
	m_CurrentSizeText[0] = '\0';
	m_IncludedTimeText[0] = '\0';
	m_IncludedSizeText[0] = '\0';
	m_TotalTimeText[0] = '\0';
	m_TotalSizeText[0] = '\0';
	m_CurrentFileFullName[0] = '\0';
	m_CurrentFileName = NULL;

	m_currentGOPVidPid = -1;
	m_currentGOPStart = 0;
	m_currentGOPEnd = -1;
	m_currentGOPEndValid = false;

	m_activeBuffer = NULL;
	m_pdlgBuffers[0].bm.buffer = NULL;
	m_pdlgBuffers[1].bm.buffer = NULL;

	m_scanBufferSizeX = 16*1024*1024;
	m_scanBufferSize = 0;
	m_scanBufferX = NULL;
	m_scanBuffer = NULL;
	m_scanBufferAlignment = 512;	// non disk cache file access requires sector aligned access
	m_scanBufferOffsetBytes = -1;
	m_scanBufferUsedBytes = 0;
	m_scanRange = 6*1024*1024; // increased size for QAM recordings
}

void CHDTVtoMPEG2Dlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CHDTVtoMPEG2Dlg)
	DDX_Control(pDX, IDC_BRIGHTNESS_SLIDER, m_brightnessSlider);
	DDX_Control(pDX, IDC_CHANNEL, m_ChannelCombo);
	DDX_Control(pDX, IDC_CONTRAST_SLIDER, m_contrastSlider);
	DDX_Control(pDX, IDC_EXTENSION, m_Extension);
	DDX_Control(pDX, IDC_FILETYPE, m_FiletypeCombo);
	DDX_Control(pDX, IDC_INPUTFILES, m_InputFiles);
	DDX_Control(pDX, IDC_OUTPUTFILE, m_OutputFile);
	DDX_Control(pDX, IDC_PIDS, m_PIDCombo );
	DDX_Control(pDX, IDC_REGION_SLIDER, m_TimeSlider);
	DDX_Text(pDX, IDC_AUDIOPID, m_AudioPIDStr);
	DDX_Text(pDX, IDC_MAXFILESIZE, m_MaxFileSize);
	DDV_MinMaxInt(pDX, m_MaxFileSize, 1, 99999);
	DDX_Text(pDX, IDC_VIDEOPID, m_VideoPIDStr);
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CHDTVtoMPEG2Dlg, CDialog)
	//{{AFX_MSG_MAP(CHDTVtoMPEG2Dlg)
	ON_WM_DESTROY()
	ON_WM_DROPFILES()
	ON_WM_HSCROLL()
	ON_WM_LBUTTONDOWN()
	ON_WM_MOUSEWHEEL()
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	ON_WM_SYSCOMMAND()

	ON_BN_CLICKED(IDC_ADD, OnAdd)
	ON_BN_CLICKED(IDC_BEGIN_EXCLUDE, OnBeginExclude)
	ON_BN_CLICKED(IDC_BEGIN_INCLUDE, OnBeginInclude)
	ON_BN_CLICKED(IDC_CHANNEL, OnChannelCheckBox )
	ON_BN_CLICKED(IDC_CHECK_PROGRESS_IMAGE, OnCheckProgressImage)
	ON_BN_CLICKED(IDC_CLEAR, OnClear)
	ON_BN_CLICKED(IDC_CLEAR_ALL_REGIONS, OnClearAllRegions)
	ON_BN_CLICKED(IDC_CLIPS, OnBnClickedClips)
	ON_BN_CLICKED(IDC_CONVERT, OnConvert)
	ON_BN_CLICKED(IDC_DELETE, OnDelete)
	ON_BN_CLICKED(IDC_NEXT_REGION, OnNextRegion)
	ON_BN_CLICKED(IDC_MOVE_REGION_BACK, OnMoveRegionBack)
	ON_BN_CLICKED(IDC_MOVE_REGION_FORWARD, OnMoveRegionForward)
	ON_BN_CLICKED(IDC_OUTPUTFILE_BROWSE, OnOutputfileBrowse)
	ON_BN_CLICKED(IDC_REMOVE_REGION, OnRemoveRegion)
	ON_BN_CLICKED(IDC_PEEK_AT_FILE, OnPeekAtFile)
	ON_BN_CLICKED(IDC_PIDS, OnPidsCheckBox )
	ON_BN_CLICKED(IDC_PREV_REGION, OnPrevRegion)
	ON_BN_CLICKED(IDC_SCAN, OnScan)
	ON_BN_CLICKED(IDC_SEEK_BACK1, OnSeekBack1)
	ON_BN_CLICKED(IDC_SEEK_BACK2, OnSeekBack2)
	ON_BN_CLICKED(IDC_SEEK_BACK3, OnSeekBack3)
	ON_BN_CLICKED(IDC_SEEK_BEGIN, OnSeekBegin)
	ON_BN_CLICKED(IDC_SEEK_END, OnSeekEnd)
	ON_BN_CLICKED(IDC_SEEK_FORWARD1, OnSeekForward1)
	ON_BN_CLICKED(IDC_SEEK_FORWARD2, OnSeekForward2)
	ON_BN_CLICKED(IDC_SEEK_FORWARD3, OnSeekForward3)

	ON_CBN_SELCHANGE(IDC_CHANNEL, OnSelchangeChannel)
	ON_CBN_SELCHANGE(IDC_FILETYPE, OnSelchangeFiletype)

	ON_EN_CHANGE(IDC_EXTENSION, OnChangeExtension)
	ON_EN_CHANGE(IDC_MAXFILESIZE, OnChangeMaxFileSize)

	ON_LBN_SELCHANGE(IDC_INPUTFILES, OnSelchangeInputfiles)
	ON_BN_CLICKED(IDC_BEGIN_CUT, OnBeginCut)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()



/////////////////////////////////////////////////////////////////////////////
// CHDTVtoMPEG2Dlg message handlers

BOOL CHDTVtoMPEG2Dlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	// find all referenced dialog items, to save on constantly looking them up later
	m_pCheckProgressImage = static_cast<CButton*>(GetDlgItem(IDC_CHECK_PROGRESS_IMAGE));
	m_pConvert = GetDlgItem(IDC_CONVERT);
	m_pDelete = GetDlgItem(IDC_DELETE);
	m_pClear = GetDlgItem(IDC_CLEAR);
	m_pChannel = GetDlgItem(IDC_CHANNEL);
	m_pTxtResolution = GetDlgItem(IDC_TXT_RESOLUTION);
	m_pTxtAspectRatio = GetDlgItem(IDC_TXT_ASPECT_RATIO);
	m_pTxtFrameRate = GetDlgItem(IDC_TXT_FRAME_RATE);
	m_pTxtBitRate = GetDlgItem(IDC_TXT_BITRATE);
	m_pCurrentTime = GetDlgItem(IDC_CURRENT_TIME);
	m_pCurrentSize = GetDlgItem(IDC_CURRENT_SIZE);
	m_pIncludedTime = GetDlgItem(IDC_INCLUDED_TIME);
	m_pIncludedSize = GetDlgItem(IDC_INCLUDED_SIZE);
	m_pTotalTime = GetDlgItem(IDC_TOTAL_TIME);
	m_pTotalSize = GetDlgItem(IDC_TOTAL_SIZE);
	m_pCurrentFile = GetDlgItem(IDC_CURRENT_FILE);
	m_pBeginInclude = GetDlgItem(IDC_BEGIN_INCLUDE);
	m_pBeginExclude = GetDlgItem(IDC_BEGIN_EXCLUDE);
	m_pRemoveRegion = GetDlgItem(IDC_REMOVE_REGION);
	m_pClearAllRegions = GetDlgItem(IDC_CLEAR_ALL_REGIONS);
	m_pScan = GetDlgItem(IDC_SCAN);
	m_pPrevRegion = GetDlgItem(IDC_PREV_REGION);
	m_pNextRegion = GetDlgItem(IDC_NEXT_REGION);
	m_pMoveRegionBack = GetDlgItem(IDC_MOVE_REGION_BACK);
	m_pMoveRegionForward = GetDlgItem(IDC_MOVE_REGION_FORWARD);
	m_pRegionSlider = GetDlgItem(IDC_REGION_SLIDER);
	m_pSeekBegin = GetDlgItem(IDC_SEEK_BEGIN);
	m_pSeekBack3 = GetDlgItem(IDC_SEEK_BACK3);
	m_pSeekBack2 = GetDlgItem(IDC_SEEK_BACK2);
	m_pSeekBack1 = GetDlgItem(IDC_SEEK_BACK1);
	m_pSeekForward1 = GetDlgItem(IDC_SEEK_FORWARD1);
	m_pSeekForward2 = GetDlgItem(IDC_SEEK_FORWARD2);
	m_pSeekForward3 = GetDlgItem(IDC_SEEK_FORWARD3);
	m_pSeekEnd = GetDlgItem(IDC_SEEK_END);
	m_pPreviewFrame = GetDlgItem(IDC_PREVIEW_FRAME);
	m_pRegionFrame = GetDlgItem(IDC_REGION_FRAME);
	m_pInputFiles = GetDlgItem( IDC_INPUTFILES );
	m_pOutputFile = GetDlgItem (IDC_OUTPUTFILE );
	m_pBrightnessSlider = GetDlgItem(IDC_BRIGHTNESS_SLIDER);
	m_pContrastSlider = GetDlgItem(IDC_CONTRAST_SLIDER);
	m_pExtension = GetDlgItem(IDC_EXTENSION );
	m_pPidsCombo = GetDlgItem( IDC_PIDS );
	m_pMaxFileSize = GetDlgItem( IDC_MAXFILESIZE );
	m_pVideoPID = GetDlgItem( IDC_VIDEOPID );
	m_pAudioPID = GetDlgItem( IDC_AUDIOPID );
	m_pCut = GetDlgItem( IDC_BEGIN_CUT );

	// Add "About..." menu item to system menu.

	// IDM_ABOUTBOX must be in the system command range.
	ASSERT((IDM_ABOUTBOX & 0xFFF0) == IDM_ABOUTBOX);
	ASSERT(IDM_ABOUTBOX < 0xF000);

	CMenu* pSysMenu = GetSystemMenu(FALSE);
	if (pSysMenu != NULL)
	{
		CString strConfigureMenu;
		strConfigureMenu.LoadString( IDS_CONFIGURE );
		if( !strConfigureMenu.IsEmpty() )
		{
			pSysMenu->AppendMenu( MF_SEPARATOR );
			pSysMenu->AppendMenu( MF_STRING, IDM_CONFIGURE, strConfigureMenu );
		}

		CString strAboutMenu;
		strAboutMenu.LoadString(IDS_ABOUTBOX);
		if (!strAboutMenu.IsEmpty())
		{
			pSysMenu->AppendMenu(MF_SEPARATOR);
			pSysMenu->AppendMenu(MF_STRING, IDM_ABOUTBOX, strAboutMenu);
		}
	}

	// Set the icon for this dialog.  The framework does this automatically
	//  when the application's main window is not a dialog
	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon

	// default settings
	GetPrivateProfileString("Settings", "ShowProgressImage", "1", temp, 255, inipath);
	m_pCheckProgressImage->SetCheck(atol(temp) != 0);

	// add filetypes
	for (int i = ft_MIN; i < ft_MAX; i++)
		m_FiletypeCombo.AddString(filetypeNames[i]);
	GetPrivateProfileString("Settings", "FileType", "0", temp, 255, inipath);
	m_Filetype = (Filetype)atol(temp);
	if ((m_Filetype < ft_MIN) || (m_Filetype >= ft_MAX))
		m_Filetype = ft_MIN;
	m_FiletypeCombo.SetCurSel(m_Filetype);

	char key[64];
	sprintf(key, "ExtType%d", m_Filetype);
	GetPrivateProfileString("Settings", key, default_extensions[m_Filetype], m_ExtensionStr, 255, inipath);
	m_Extension.SetWindowText(m_ExtensionStr);

	// NULL PID INI setting
	GetPrivateProfileString( "Settings", "KeepNULLs", "0", temp, 2, inipath );
	if( temp[0] == '0' )
		m_KeepNULLs = false;
	else
		m_KeepNULLs = true;

	// PSIP PID INI setting
	GetPrivateProfileString( "Settings", "KeepPSIP", "0", temp, 2, inipath );
	if( temp[0] == '0' )
		m_KeepPSIP = false;
	else
		m_KeepPSIP = true;

	// Contrast INI setting
	GetPrivateProfileString( "Settings", "Contrast", "6", temp, 5, inipath );
	m_Contrast = atoi( temp );

	// Brightness INI setting
	GetPrivateProfileString( "Settings", "Brightness", "1", temp, 5, inipath );
	m_Brightness = atoi( temp );

	pPIDDeleteList = new CWordArray();
	pHiddenPidDeleteList = new CWordArray();
	pProgramMapPIDs = new CWordArray();
	pPCRPIDs = new CWordArray();

	// pid and mapping infomation
	m_pPidInfoList = new CPtrArray();

	// saved PAT and PMT packets data
	PatPmtPackets.numOfPMTPackets = 0;

	m_Clips = false;

	GetConfigSettings();

	// init bitmap stuff
	m_doPreview = false;
	if (m_pPreviewFrame)
	{
		m_doPreview = true;

		m_pPreviewFrame->GetWindowRect(m_previewRect);
		ScreenToClient(m_previewRect);

		m_BMIH.biSize = sizeof(BITMAPINFOHEADER);
		m_BMIH.biWidth = 240;	// oh look, a hard coded size
		m_BMIH.biHeight = 135;	// 240x135 = 1920x1080 / 8 (1 pixel per block)
		m_BMIH.biPlanes = 1;
		m_BMIH.biBitCount = 24;
		m_BMIH.biCompression = BI_RGB;
		m_BMIH.biXPelsPerMeter = 0;
		m_BMIH.biYPelsPerMeter = 0;
		m_BMIH.biClrUsed = 0;
		m_BMIH.biClrImportant = 0;

		DWORD dwBytes = ((DWORD) m_BMIH.biWidth * m_BMIH.biBitCount) / 32;
		if (((DWORD) m_BMIH.biWidth * m_BMIH.biBitCount) % 32)
			dwBytes++;
		dwBytes *= 4;
		m_BMIH.biSizeImage = 4 * dwBytes * m_BMIH.biHeight;

		// set up buffers if needed

		// if buffers already exists, are they the wrong size?
		if (m_pdlgBuffers[0].bm.buffer &&
				((m_pdlgBuffers[0].bm.width != (uint)m_BMIH.biWidth) ||
					(m_pdlgBuffers[0].bm.height != (uint)m_BMIH.biHeight)))
		{
			delete []pdlg_buffers[0].bm.buffer;
			pdlg_buffers[0].bm.buffer = NULL;
		}

		if (m_pdlgBuffers[1].bm.buffer &&
				((m_pdlgBuffers[1].bm.width != (uint)m_BMIH.biWidth) ||
					(m_pdlgBuffers[1].bm.height != (uint)m_BMIH.biHeight)))
		{
			delete []pdlg_buffers[1].bm.buffer;
			pdlg_buffers[1].bm.buffer = NULL;
		}

		// do we need to setup & allocate the buffers?
		if (!m_pdlgBuffers[0].bm.buffer)
		{
			m_pdlgBuffers[0].bm.width = m_BMIH.biWidth;
			m_pdlgBuffers[0].bm.height = m_BMIH.biHeight;
			m_pdlgBuffers[0].bm.stride = (m_BMIH.biWidth * m_BMIH.biBitCount)>>3;
			m_pdlgBuffers[0].bm.size = m_BMIH.biSizeImage;

			m_pdlgBuffers[0].bm.buffer = new uchar[m_BMIH.biSizeImage];
		}

		if (!m_pdlgBuffers[1].bm.buffer)
		{
			m_pdlgBuffers[1].bm.width = m_BMIH.biWidth;
			m_pdlgBuffers[1].bm.height = m_BMIH.biHeight;
			m_pdlgBuffers[1].bm.stride = (m_BMIH.biWidth * m_BMIH.biBitCount)>>3;
			m_pdlgBuffers[1].bm.size = m_BMIH.biSizeImage;

			m_pdlgBuffers[1].bm.buffer = new uchar[m_BMIH.biSizeImage];
		}

		if (m_pdlgBuffers[0].bm.buffer && m_pdlgBuffers[1].bm.buffer)
		{
			// clear the buffers
			memset(m_pdlgBuffers[0].bm.buffer, 0, m_BMIH.biSizeImage);
			memset(m_pdlgBuffers[1].bm.buffer, 0, m_BMIH.biSizeImage);

			// tag them as available to the conversion thread
			m_pdlgBuffers[0].rendered = 0;
			m_pdlgBuffers[1].rendered = 0;
		}
		else
		{
			m_doPreview = false;
		}

		m_activeBuffer = NULL;
	}

	m_doRegion = false;
	if (m_pRegionFrame)
	{
		m_doRegion = true;

		m_pRegionFrame->GetWindowRect(m_regionRect);
		ScreenToClient(m_regionRect);
	}

	//
	m_scanBufferX = new unsigned char[(int)m_scanBufferSizeX];
	m_scanBuffer = new unsigned char[(int)m_scanBufferSizeX];
	SetUpScanBuffer();

	// debug stuff
#ifdef _DEBUG
	CWnd *pr = GetDlgItem(IDC_PEEK_AT_FILE);
	if (pr)
		pr->ShowWindow(true);
#endif

#ifdef FINAL_RELEASE
	m_pScan->ShowWindow(false);
#endif

	m_canLargePreview = ChooseIDCT();

	SetVisibleStates();

	ConfigureRegionsUI();
	UpdateRegionsUI();

	// process the command line if there was a file specified
	CHDTVtoMPEG2App * theApp = (CHDTVtoMPEG2App*)AfxGetApp();

	// get the file specified on the command line
	CString theCmdLine;
	theCmdLine = theApp->m_lpCmdLine;

	if( theCmdLine.GetLength() > 0 )
		ProcessCmdLine( theCmdLine );

	return TRUE;  // return TRUE  unless you set the focus to a control
}

// process the string passed on the command line
//  used for drag and drop support on the desktop icon 
// NOTE: the parse routine will fail if the command line
//       has both quoted and non-quoted names
void CHDTVtoMPEG2Dlg::ProcessCmdLine( CString sCmdLine )
{
	CString name;
	CString sFileName;
	CString msg;
	int index;

	sFileName = sCmdLine;
	// remove the qoutes
	sFileName.Remove( '"' );

	index = sFileName.ReverseFind( '\\' );
	name = sFileName.Right( sFileName.GetLength() - index - 1 );

	// check the file exention
	index = name.ReverseFind( '.' );
	CString ext = name.Right( name.GetLength() - index - 1 );
/*
msg.Format("sCmdLine = [%s]", sCmdLine );
MessageBox(msg);
*/

	if( ext == "tri" || ext == "TRI" )
	{
		if( !ProcessTRIFile( sFileName ) )
			return;
	}

	// parse the command line string
	// known transport stream files only
	else if( ext == "ts"   || ext == "TS"   ||
		     ext == "tp"   || ext == "TP"   ||
			 ext == "trs"  || ext == "TRS"  ||
			 ext == "trp"  || ext == "TRP"  ||
			 ext == "adtv" || ext == "ADTV" ||
			 ext == "m2t"  || ext == "M2T"  ||
			 ext == "rec"  || ext == "REC"  ||
			 ext == "mpg"  || ext == "MPG" )
	{
		char ch;
		CString name;

		int i = 0;
		bool quoted = false;
		bool start = false;

//MessageBox("About to start while loop");
		while( i < sCmdLine.GetLength() )
		{
			ch = sCmdLine.GetAt( i );

			if( ch == ' ' && quoted && !start ) // ignore blank characters between quoted names
			{
				++i;
				continue;
			}
	
			if( ch == ' ' && !quoted ) // non quoted name seperated by spaces
			{
//MessageBox("non quoted name seperated by spaces");
				if( start ) // this is the end of the name
				{
/*
msg.Format("name = %s", name );
MessageBox( msg );
*/
					if( GetFileByteSize( name ) == 0 )
					{
						CString msg;
						msg.Format( "Discarding 0 byte file:\n%s", name );
						MessageBox( msg );
						i++;
						name = "";
						start = false;
						continue;
					}

					AddFile( name );
					name = "";
					start = false;
					i++;
					continue;
				}

				else // this is the start of a new name
				{
					start = true;
					i++;
					continue;
				}
			}

			if( ch == '"' ) // quoted name, seperated by spaces
			{
//MessageBox("quoted name, seperated by spaces");
				if( start ) // this is the end of the quoted name
				{
					if( GetFileByteSize( name ) == 0 )
					{
						CString msg;
						msg.Format( "Discarding 0 byte file:\n%s", name );
						MessageBox( msg );
						i++;
						name = "";
						start = false;
						continue;
					}

					AddFile( name );
					name = "";
					start = false;
					i++;
					continue;
				}
				else // this is the start of a new name
				{
					start = true;
					quoted = true;
					i++;
					continue;
				}
			}

			else // continuation of name
			{
				name += ch;
				start = true;
				i++;
			}
		}// end while( i < sCmdLine.GetLength() )

/*
MessageBox("Out of while statment");

msg.Format("i = %d", i );
MessageBox(msg);

msg.Format("start = %d", start );
MessageBox( msg );
*/

		if( start )
		{
			if( GetFileByteSize( name ) == 0 )
			{
				msg.Format( "Discarding 0 byte file:\n%s", name );
				MessageBox( msg );
			}
			else
				AddFile( name ); // used when drag\drop on desktop icon
		}
	}

	else // not a valid file to parse
	{
		MessageBox( "File must have a valid extension!", "HDTVtoMPEG2 - Invalid file type!", MB_OK | MB_ICONERROR );
		return;
	}

	if( m_InputFiles.GetCount() != 0 )
	{
		CreateOutputFilename();

		SetVisibleStates();

		ConfigureRegionsUI();
		UpdateRegionsUI();
		EmptyScanBuffer();


		int n = m_InputFiles.GetTextLen( 0 );
		m_InputFiles.GetText( 0, sFileName.GetBuffer( n ) );

		// save the input file location in the ini file
		CString sPath = sFileName;
		index = sPath.ReverseFind( '\\' );
		sPath = sPath.Left( index + 1 );
		m_InputPath = sPath;
		WritePrivateProfileString( "Settings", "PathIn", m_InputPath, inipath );

		if( m_tsiFile.GetFilePath().IsEmpty() && !sFileName.IsEmpty() )
			CreateTSIFileName( sFileName );
	}
}

void CHDTVtoMPEG2Dlg::SetVisibleStates(void) 
{
	int bCon = FALSE;
	int bDel = FALSE;
	int bClr = FALSE;
	int bChan = FALSE;
		
	if (m_InputFiles.GetCount() > 0)
	{
		bCon = TRUE;
		bClr = TRUE;
	}

	if (m_InputFiles.GetSelCount() > 0)
	{
		bDel = TRUE;
	}

	if (m_ChannelCombo.GetCount() > 0)
	{
		bChan = TRUE;
	}
		
	m_pConvert->EnableWindow(bCon);
	m_pDelete->EnableWindow(bDel);
	m_pClear->EnableWindow(bClr);
	m_pChannel->EnableWindow(bChan);
	m_pPidsCombo->EnableWindow(bChan);

	m_pTxtResolution->ShowWindow(bChan);
	m_pTxtAspectRatio->ShowWindow(bChan);
	m_pTxtFrameRate->ShowWindow(bChan);
	m_pTxtBitRate->ShowWindow(bChan);
}

void CHDTVtoMPEG2Dlg::OnDestroy() 
{
	// save the region markers before quiting
	SaveRegionInfo();

	if (m_scanBufferX)
		delete []m_scanBufferX;
	if (m_scanBuffer)
		delete []m_scanBuffer;

	// delete region editor preview buffers
	if (m_pdlgBuffers[0].bm.buffer)
		delete []m_pdlgBuffers[0].bm.buffer;
	m_pdlgBuffers[0].bm.buffer = NULL;

	if (m_pdlgBuffers[1].bm.buffer)
		delete []m_pdlgBuffers[1].bm.buffer;
	m_pdlgBuffers[1].bm.buffer = NULL;

	// delete progress dialog preview buffers
	if (pdlg_buffers[0].bm.buffer)
		delete []pdlg_buffers[0].bm.buffer;
	pdlg_buffers[0].bm.buffer = NULL;

	if (pdlg_buffers[1].bm.buffer)
		delete []pdlg_buffers[1].bm.buffer;
	pdlg_buffers[1].bm.buffer = NULL;

	if( pPIDDeleteList )
	{	
		pPIDDeleteList->RemoveAll();
		delete pPIDDeleteList;
	}
	if( pProgramMapPIDs )
	{
		pProgramMapPIDs->RemoveAll();
		delete pProgramMapPIDs;
	}
	if( pPCRPIDs )
	{
		pPCRPIDs->RemoveAll();
		delete pPCRPIDs;
	}
	if( pHiddenPidDeleteList )
	{
		pHiddenPidDeleteList->RemoveAll();
		delete pHiddenPidDeleteList;
	}
	if( m_pPidInfoList )
	{
		Pid_Info *pPidInfo;
		for( int i = 0; i < m_pPidInfoList->GetCount(); i++ )
		{
			pPidInfo = (Pid_Info*)m_pPidInfoList->GetAt( i );
			delete pPidInfo;
		}

		m_pPidInfoList->RemoveAll();
		delete m_pPidInfoList;
	}

	if( Stream_Header.buffer )
		delete []Stream_Header.buffer;
	if( PatPmtPackets.pPAT_Packet )
		delete []PatPmtPackets.pPAT_Packet;

    for( int i = 0; i < PatPmtPackets.numOfPMTPackets; i++ )
			delete []PatPmtPackets.pPMT_Packet[i];

	// save Contrast and Brightness settings
	char temp[5];

	m_Contrast = m_contrastSlider.GetPos();
	sprintf( temp, "%d", m_Contrast );
	WritePrivateProfileString( "Settings", "Contrast", temp, inipath );

	m_Brightness = m_brightnessSlider.GetPos();
	sprintf( temp, "%d", m_Brightness );
	WritePrivateProfileString( "Settings", "Brightness", temp, inipath );

	CDialog::OnDestroy();
}

void CHDTVtoMPEG2Dlg::OnSysCommand(UINT nID, LPARAM lParam)
{
	if( nID == IDM_ABOUTBOX )
	{
		CAboutDlg dlgAbout;
		dlgAbout.DoModal();
	}
	else if( nID == IDM_CONFIGURE )
	{
		CConfigureDlg dlgConfigure( 0, inipath );
		if( dlgConfigure.DoModal() == IDOK )
			GetConfigSettings();
	}
	else
	{
		CDialog::OnSysCommand(nID, lParam);
	}
}

// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.

void CHDTVtoMPEG2Dlg::OnPaint() 
{
	if (IsIconic())
	{
		CPaintDC dc(this); // device context for painting

		SendMessage(WM_ICONERASEBKGND, (WPARAM) dc.GetSafeHdc(), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		DrawPreview();
		DrawRegions();
		CDialog::OnPaint();
	}
}

void CHDTVtoMPEG2Dlg::DrawPreview(void)
{
	// draw preview
	if (m_doPreview && m_activeBuffer)
	{
		CDC *pDC = GetDC();

		// do a stretch according to aspect ratio

		// note: m_previewRect is placed in the dialog to be as close as
		// possible to 135 pixels in height (which is 1080 / 8)
		// and the width is large enough to cope with the 2.21 ratio

		float widthRatio = 16.0f/9.0f;	// default
		switch (m_activeBuffer->aspectRatio)
		{
		case 1: widthRatio = 1.0f/1.0f; break;
		case 2: widthRatio = 4.0f/3.0f; break;
		case 3: widthRatio = 16.0f/9.0f; break;
		case 4: widthRatio = 2.21f/1.0f; break;
		}

		int height = 135;	//m_previewRect.Height();
		int width = (int)((float)height * widthRatio);
		//	calculate appropriate top left
		int left = m_previewRect.left + ((m_previewRect.Width() - width)>>1);
		int top = m_previewRect.top;// + ((m_previewRect.Height() - m_activeBuffer->bm.height)>>1);

		COLORREF cr = GetSysColor(COLOR_3DFACE);

		CRect r;
		r = m_previewRect;
		r.right = left;
		pDC->FillSolidRect(r, cr);

		r = m_previewRect;
		r.left = left + width;
		pDC->FillSolidRect(r, cr);

		pDC->SetStretchBltMode(COLORONCOLOR);
		StretchDIBits(pDC->GetSafeHdc(), left, top, width, height,
			0, m_activeBuffer->bm.height-m_activeBuffer->renderedHeight, m_activeBuffer->renderedWidth, m_activeBuffer->renderedHeight,
			m_activeBuffer->bm.buffer, (LPBITMAPINFO) &m_BMIH, DIB_RGB_COLORS, SRCCOPY);

		ReleaseDC( pDC );
	}
}

void CHDTVtoMPEG2Dlg::DrawRegions(__int64 marker)
{
	// draw regions bar
	TSFileInfo *ti = m_InputList.GetHead();
	if (m_doRegion && ti)
	{
		CDC *pDC = GetDC();

		CRect r;  // Include/Exlcude bar
		CRect r2; // Include/Exclude marker
		CRect r3; // Cut marker

		r = m_regionRect;
		r2 = r;
		r2.bottom = r2.top + (r2.Height()*2)/3;
		r3 = r;

		int zeroLeft = r.left;
		int totalWidth = m_regionRect.Width();

		int fileCount = 0;
		RegionType rt = rt_include;
		while (ti)
		{
			int level = 255;
			if ((fileCount++)%2)
				level = 200;

			TSRegionInfo *tri = ti->m_head;

			r.left = zeroLeft + int((__int64(totalWidth) * ti->m_totalBytesPrior) / m_InputList.TotalBytes());
			if (tri)
				r.right = zeroLeft + int((__int64(totalWidth) * (ti->m_totalBytesPrior + tri->m_offset)) / m_InputList.TotalBytes());
			else
				r.right = zeroLeft + int((__int64(totalWidth) * (ti->m_totalBytesPrior + ti->m_fileSize)) / m_InputList.TotalBytes());

			COLORREF cr;
			if( rt == rt_include || rt == rt_cut )
				cr = RGB(0,level,0);
			else
				cr = RGB(level,0,0);

			pDC->FillSolidRect(r, cr);
				
			while (tri)
			{
				r.left = r.right;
				if (tri->m_next)
					r.right = zeroLeft + int((__int64(totalWidth) * (ti->m_totalBytesPrior + tri->m_next->m_offset)) / m_InputList.TotalBytes());
				else
					r.right = zeroLeft + int((__int64(totalWidth) * (ti->m_totalBytesPrior + ti->m_fileSize)) / m_InputList.TotalBytes());

				rt = tri->m_type;

				COLORREF cr;
				if( rt == rt_include || rt == rt_cut )
					cr = RGB(0,level,0);
				else
					cr = RGB(level,0,0);
/*
				else if( rt == rt_exclude )
					cr = RGB(level,0,0);
				else
				{

					TSRegionInfo *t = tri;
					RegionType rType;

					while( t != NULL && t->m_type == rt_cut )
						t = t->m_prev;

					if( t == NULL )
						rType = rt_include;
					else
						rType = t->m_type;

					if( rType == rt_include )
						cr = RGB( 0, level, 0 );
					else
						cr = RGB( level, 0, 0 );
				}
*/
				if( rt != rt_cut )
				{
					pDC->FillSolidRect(r, cr);

					r2.left = r.left;
					r2.right = r2.left +1;

					pDC->FillSolidRect(r2, 0);
				}
				else
				{
					// draw the "cut" marker
					r3.left = r.left;
					r3.right = r3.left + 1;
					pDC->FillSolidRect( r3, RGB(0, 0, level));

/*					
					int x_start, x_end;
					int y_start, y_end;

					x_start = r3.left;
					y_start = r3.top;
					x_end = x_start - ( r3.Height() / 4 );
					y_end = y_start - ( r3.Height() / 4 );

					pDC->MoveTo( x_start, y_start );
					pDC->LineTo( x_end, y_end );

					x_start = r3.right;
					y_start = r3.bottom;
					x_end = x_start + ( r3.Height() / 4 );
					y_end = y_start + ( r3.Height() / 4 );

					pDC->MoveTo( x_start, y_start );
					pDC->LineTo( x_end, y_end );
*/

					// draw the rest of the green\red color bar
					r.left += 1;
					pDC->FillSolidRect(r, cr);
				}

				tri = tri->m_next;
			}

			ti = ti->m_next;
		}

		if (marker >= 0)
		{
			r.left = zeroLeft + int((__int64(totalWidth) * marker) / m_InputList.TotalBytes());
			r.right = r.left + 1;

			pDC->FillSolidRect(r, RGB(0,0,255));
		}

		ReleaseDC( pDC );
	}
}

// The system calls this to obtain the cursor to display while the user drags
//  the minimized window.
HCURSOR CHDTVtoMPEG2Dlg::OnQueryDragIcon()
{
	return (HCURSOR) m_hIcon;
}

void CHDTVtoMPEG2Dlg::CreateOutputFilename(void)
{
	TSFileInfo *tsi;

	// Get output filename
	if (m_InputFiles.GetCount() > 0)
	{
		UpdateData(TRUE);

		CString name;
		char buf[256];

		tsi = m_InputList.GetHead();
		name = tsi->m_fileName;

		GetPrivateProfileString("Settings", "PathOut", "", buf, 255, inipath);
		if (buf[0])
		{
			char *p = strrchr(buf, '\\');
			if (p)
			{
				p++;
				*p = '\0';
				char *q = (char*)strrchr(name, '\\');
				if (q)
				{
					q++;
					strcpy(p, q);
				}
			}
			else
				buf[0] = '\0';
		}
		if (!buf[0])
		{
			SaveOutputFilePath(name);
			strcpy(buf, name);
		}

// Cris - fix this to support TS, etc
		char *p = buf + strlen(buf) - 1;
		while (p != buf && *p >= '0' && *p <= '9')
			p--;
		if (p != buf && *p == '.')
			p--;
//		if ((p - 2) > buf && strnicmp(p - 2, ".ts", 3) == 0)
		if ((p - 2) > buf && _strnicmp(p - 2, ".ts", 3) == 0)
			p -= 2;
//		else if ((p - 2) > buf && strnicmp(p - 2, ".tp", 3) == 0)
		else if ((p - 2) > buf && _strnicmp(p - 2, ".tp", 3) == 0)
			p -= 2;
//		else if ((p - 3) > buf && strnicmp(p - 2, ".trs", 3) == 0)
		else if ((p - 3) > buf && _strnicmp(p - 2, ".trs", 3) == 0)
			p -= 3;
//		else if ((p - 3) > buf && strnicmp(p - 2, ".trp", 3) == 0)
		else if ((p - 3) > buf && _strnicmp(p - 2, ".trp", 3) == 0)
			p -= 3;
//		else if ((p - 4) > buf && strnicmp(p - 4, ".adtv", 5) == 0)
		else if ((p - 4) > buf && _strnicmp(p - 4, ".adtv", 5) == 0)
			p -= 4;
		*p = '\0';

		strcat(buf, m_ExtensionStr);

		m_OutputFile.SetWindowText(buf);
	}
}

void CHDTVtoMPEG2Dlg::RemoveOutputFilenameExtension(char *ext)
{
	CString name;
	char buf[256];

	m_OutputFile.GetWindowText(name);
	int l = strlen(name);
	if (l > 0)
	{
		strcpy(buf, name);
		char *t = buf + l - strlen(ext);
//		if (!stricmp(t, ext))
		if( !_stricmp( t, ext ))
		{
			*t = '\0';

			m_OutputFile.SetWindowText(buf);
		}
	}
}

void CHDTVtoMPEG2Dlg::AddOutputFilenameExtension(char *ext)
{
	CString name;
	char buf[256];

	m_OutputFile.GetWindowText(name);

	if (strlen(name) > 0)
	{
		strcpy(buf, name);
		strcat(buf, m_ExtensionStr);
		m_OutputFile.SetWindowText(buf);
	}
}

void CHDTVtoMPEG2Dlg::AddFile(const char *fileName)
{
	HANDLE f = INVALID_HANDLE_VALUE;
	uchar *destAligned = (uchar*)(((int)m_scanBufferX + m_scanBufferAlignment - 1) & (~((__int64)m_scanBufferAlignment - 1)));
	bool filled = false;

	int inputCount = m_InputFiles.GetCount();

	// Scan for channels
	if( inputCount == 0 )
	{
		// grab a chunk of the file and scan it...
		f = CreateFile(fileName, GENERIC_READ, FILE_SHARE_READ | FILE_SHARE_WRITE, NULL, OPEN_EXISTING,
								FILE_ATTRIBUTE_NORMAL | FILE_FLAG_NO_BUFFERING | FILE_FLAG_RANDOM_ACCESS, NULL);
		if (f != INVALID_HANDLE_VALUE)
		{
			DWORD r;
			
			if (!ReadFile(f, destAligned, (DWORD)m_scanBufferSize, &r, NULL))
			{
				r = 0;
			}
			else
			{
				filled = true;
				scan_channel_info(destAligned, r, MAXCHANNELS, m_Channels, m_NumChannels,
					              pPIDDeleteList, pHiddenPidDeleteList );
			}
		}

		if( m_NumChannels == 0 )
		{
			MessageBox( "Could not find a Channel!", "Warning" );

			if( f )
				CloseHandle(f);

			return;
		}

		char chbuf[80];
		int index = 0;
		for (int ch = 0; ch < m_NumChannels; ch++)
		{
			sprintf(chbuf, "%s %d.%d", m_Channels[ch].name, m_Channels[ch].majorChannel, m_Channels[ch].minorChannel);
			m_ChannelCombo.AddString(chbuf);

			// populate our PID's combobox
			sprintf( chbuf, "0x%X(video)", (WORD)m_Channels[ch].videopid );
			if( m_PIDCombo.FindStringExact( -1, chbuf ) == CB_ERR )
			{
				m_PIDCombo.AddString( chbuf );
				// 1st 16 bits = pType, last 16 bits = pid
				m_PIDCombo.SetItemData( index, m_Channels[ch].videopid );
				++index;
			}

			sprintf( chbuf, "0x%X(audio)", (WORD)m_Channels[ch].audiopid );
			m_PIDCombo.AddString( chbuf );
			// 1st 16 bits = pType, last 16 bits = pid
			m_PIDCombo.SetItemData( index, m_Channels[ch].audiopid );
			++index;
		}

		// find a Channel with a horizontal size >= 1280, an HD channel
		//  make the first one we find the default Channel
		int channel = 0;
		int resolution = 0;
		for( int ch = 0; ch < m_NumChannels; ch++ )
		{
			if( m_Channels[ch].width >= 1280 )
			{
				channel = ch;
				resolution = m_Channels[ch].width;
				break;
			}
		}

		// now add the null and psip pids if we found any in the stream
		int pid = 0;
		for( int i = 0; i < pPIDDeleteList->GetCount(); i++ )
		{
			pid = pPIDDeleteList->GetAt( i );

			if( pid == 0x1ffb )
				sprintf( chbuf, "0x%X(psip)", pid );
			if( pid == 0x1fff )
				sprintf( chbuf, "0x%X(null)", pid );
			if( pid == 0x1fef )
				sprintf( chbuf, "0x%X(null)", pid );

			m_PIDCombo.AddString( chbuf );

			if( pid == 0x1ffb )
				m_PIDCombo.SetItemData( index, PID_PSIP << 16 | pid );
			if( pid == 0x1fff )
				m_PIDCombo.SetItemData( index, PID_NULL << 16 | pid );
			if( pid == 0x1fef )
				m_PIDCombo.SetItemData( index, PID_NULL << 16 | pid );

			// set state of NULL and PSIP based state stored in INI file
			if( pid == 0x1fff || pid == 0x1fef )
				m_PIDCombo.SetCheck( index, m_KeepNULLs ? TRUE : FALSE );
			if( pid == 0x1ffb )
				m_PIDCombo.SetCheck( index, m_KeepPSIP ? TRUE : FALSE );

			++index;
		}

		// Set the Channel
		if( m_ChannelCombo.GetCurSel() < 0 )
		{
			m_ChannelCombo.SetCurSel( channel );
			OnSelchangeChannel();

			// check the Channel and appropiate PIDs
			m_ChannelCombo.SetCheck( channel );
			
		}

		// select default PID
		if( m_PIDCombo.GetCurSel() < 0 )
		{
			m_PIDCombo.SetCurSel( 0 );
			OnChannelCheckBox();
		}
	}// end if( inputCount == 0 )



	// check for duplicate files
	int i;
	for( i = 0; i < inputCount; i++ )
	{
		CString temp;
		m_InputFiles.GetText( i, temp );
//		if( !stricmp( fileName, temp ))
		if( !_stricmp( fileName, temp ))
			break;
	}

	// get some info for this file...
	__int64 size = 0;
	int offset = 0;

	if (f == INVALID_HANDLE_VALUE)
		f = CreateFile(fileName, GENERIC_READ, FILE_SHARE_READ | FILE_SHARE_WRITE, NULL, OPEN_EXISTING,
							FILE_ATTRIBUTE_NORMAL | FILE_FLAG_NO_BUFFERING | FILE_FLAG_RANDOM_ACCESS, NULL);
	if (f != INVALID_HANDLE_VALUE)
	{
		if (i >= inputCount)
		{
			// get disk info
			DWORD SectorsPerCluster = 0;
			DWORD BytesPerSector = 0;
			DWORD NumberOfFreeClusters = 0;
			DWORD TotalNumberOfClusters = 0;

			char vol[8];
			char *e = (char*)strchr(fileName, ':');
			for( i = 0; fileName + i <= e; i++ )
				vol[i] = fileName[i];
			vol[i++] = '\\';
			vol[i] = '\0';

			if (!GetDiskFreeSpace(vol, &SectorsPerCluster, &BytesPerSector, &NumberOfFreeClusters, &TotalNumberOfClusters))
				_Error("GetDiskFreeSpace() failed %d\n", GetLastError());

			if (m_scanBufferAlignment < (int)BytesPerSector)
			{
				m_scanBufferAlignment = (int)BytesPerSector;
				SetUpScanBuffer();
			}

			// get file size
			DWORD hi;
			DWORD lo = GetFileSize(f, &hi);
			size = (__int64(hi) << 32) | (__int64(lo) & 0xffffffff);

			// get packet alignment offset for this file
			DWORD bytesToReadAligned = ((20*188 + m_scanBufferAlignment - 1) & (~(m_scanBufferAlignment - 1)));
			DWORD r;

			if( filled )
			{          
				r = bytesToReadAligned;
			}
			else
			{
				if (!ReadFile(f, destAligned, bytesToReadAligned, &r, NULL))
					r = 0;
			}
			offset = align_to_next_packet(destAligned, r);

			// now add to lists
			m_InputFiles.AddFile( fileName );
			m_InputList.Add(fileName, size, offset);

#ifdef _DEBUG
			// check fileCount is the same
			if (m_InputFiles.GetCount() != m_InputList.Count())
				__asm int 3;
#endif
		}
		CloseHandle(f);
	}
}

void CHDTVtoMPEG2Dlg::OnDropFiles(HDROP hDropInfo)
{
	UINT nFiles = ::DragQueryFile(hDropInfo, (UINT)-1, NULL, 0);

	// see if its a TRI file
	TCHAR szFileName[_MAX_PATH];
	::DragQueryFile( hDropInfo, 0, szFileName, _MAX_PATH );

	CString name;
	CString sFileName;
	int index;

	sFileName = szFileName;
	index = sFileName.ReverseFind( '\\' );
	name = sFileName.Right( sFileName.GetLength() - index - 1 );

	// check the file exention
	index = name.ReverseFind( '.' );
	CString ext = name.Right( name.GetLength() - index - 1 );

	if( ext == "tri" || ext == "TRI" )
	{
		if( !ProcessTRIFile( sFileName ) )
			return;
	}
	else if( ext == "ts"   || ext == "TS"   ||
		     ext == "tp"   || ext == "TP"   ||
			 ext == "trs"  || ext == "TRS"  ||
			 ext == "trp"  || ext == "TRP"  ||
			 ext == "adtv" || ext == "ADTV" ||
			 ext == "m2t"  || ext == "M2T"  ||
			 ext == "rec"  || ext == "REC"  ||
			 ext == "mpg"  || ext == "MPG" )
	{
		for (UINT iFile = 0; iFile < nFiles; iFile++)
		{
			TCHAR szFileName[_MAX_PATH];
			::DragQueryFile(hDropInfo, iFile, szFileName, _MAX_PATH);

			if( GetFileByteSize( szFileName ) == 0 )
			{
				CString msg;
				msg.Format( "Discarding 0 byte file:\n%s", szFileName );
				MessageBox( msg );
				continue;
			}

			AddFile( szFileName );
		}
	}
	else // don't know what this file is
	{
		MessageBox( "File must have a valid extension!", "HDTVtoMPEG2 - Invalid file type!", MB_OK | MB_ICONERROR );
		return;
	}

	CreateOutputFilename();

	SetVisibleStates();

	ConfigureRegionsUI();
	UpdateRegionsUI();
	EmptyScanBuffer();

	if( m_InputFiles.GetCount() != 0 )
	{
		m_InputFiles.GetText( 0, sFileName );
		if( !sFileName.IsEmpty() )
			CreateTSIFileName( sFileName );
	}

	::DragFinish(hDropInfo);
}

void CHDTVtoMPEG2Dlg::OnAdd() 
{
	CFileDialog fdlg( TRUE,
		              "*.0000",
					  NULL,
					  OFN_ALLOWMULTISELECT | OFN_ENABLESIZING | OFN_FILEMUSTEXIST | OFN_HIDEREADONLY,
		              "Transport Files|*.ts;*.tp;*.trs;*.trp;*.tps;*.ts;*.adtv;*.m2t;*.rec;*.mpg|TRI Files|*.tri|All Files|*.*||",
					  this );

	LPSTR saveFile = fdlg.m_ofn.lpstrFile;
	UINT saveMaxFile = fdlg.m_ofn.nMaxFile;
	UINT saveMaxFileTitle = fdlg.m_ofn.nMaxFileTitle;
	fdlg.m_ofn.nMaxFile = 0x4000;
	fdlg.m_ofn.lpstrFile = new char[fdlg.m_ofn.nMaxFile];
	fdlg.m_ofn.nMaxFileTitle = 0x1000;
	fdlg.m_ofn.lpstrFileTitle = new char[fdlg.m_ofn.nMaxFileTitle];

	memset( fdlg.m_ofn.lpstrFile, 0, fdlg.m_ofn.nMaxFile );
	memset( fdlg.m_ofn.lpstrFileTitle, 0, fdlg.m_ofn.nMaxFileTitle );

	char path[256];

	if( !m_InputPath.IsEmpty() )
		fdlg.m_ofn.lpstrInitialDir = m_InputPath;
	else
	{
		GetPrivateProfileString("Settings", "PathIn", "", path, 255, inipath);
		fdlg.m_ofn.lpstrInitialDir = path;
	}

	CString sName;
	if (fdlg.DoModal() == IDOK)
	{
		WritePrivateProfileString("Settings", "PathIn", fdlg.GetPathName(), inipath);

		sName = fdlg.GetFileName();

		// check the file exention
		int index = sName.ReverseFind( '.' );
		CString ext = sName.Right( sName.GetLength() - index - 1 );

		if( ext == "tri" || ext == "TRI" )// TRI file processing
		{
			if( !ProcessTRIFile( fdlg.GetPathName() ))
			{
				delete []fdlg.m_ofn.lpstrFile;
				return;
			}
		}
		else // regular file processing
		{
			POSITION pos = fdlg.GetStartPosition();
		
			CString name;
			__int64 size = 0;

			while( pos )
			{
				name = fdlg.GetNextPathName(pos);
				size = GetFileByteSize( name );

				if( size == 0 )
				{
					CString msg;
					msg.Format( "Discarding 0 byte file:\n%s", name );
					MessageBox( msg );
					continue;
				}

				if( size == -1 )
					break;

				index = name.ReverseFind( '.' );
				ext = name.Right( name.GetLength() - index - 1 );

				if( ext == "ts"   || ext == "TS"   ||
					ext == "tp"   || ext == "TP"   || 
					ext == "trs"  || ext == "TRS"  || 
					ext == "trp"  || ext == "TRP"  || 
					ext == "adtv" || ext == "ADTV" || 
					ext == "m2t"  || ext == "M2T"  ||
					ext == "rec"  || ext == "REC"  ||
					ext == "mpg"  || ext == "MPG" )

					AddFile(name);
				else
				{
					MessageBox( "File must have a valid extension!", "HDTVtoMPEG2 - Invalid file type!", MB_OK | MB_ICONERROR );
					delete []fdlg.m_ofn.lpstrFile;
					return;
				}
			}// end while( pos )
		}

		CreateOutputFilename();

		SetVisibleStates();

		ConfigureRegionsUI();
		UpdateRegionsUI();
		EmptyScanBuffer();

		if( m_InputFiles.GetCount() != 0 )
		{
			m_InputFiles.GetText( 0, sName );
			if( !sName.IsEmpty() )
				CreateTSIFileName( sName );
		}
	}

	delete []fdlg.m_ofn.lpstrFile;
	delete []fdlg.m_ofn.lpstrFileTitle;
	fdlg.m_ofn.lpstrFile = saveFile;
	fdlg.m_ofn.nMaxFile = saveMaxFile;
	fdlg.m_ofn.nMaxFileTitle = saveMaxFileTitle;
}

// get the byte size of a file
// must pass in full path and file name
__int64 CHDTVtoMPEG2Dlg::GetFileByteSize( CString sFileName )
{

	HANDLE f = INVALID_HANDLE_VALUE;
	__int64 size = 0;

	f = CreateFile( sFileName, GENERIC_READ, FILE_SHARE_READ | FILE_SHARE_WRITE, NULL, OPEN_EXISTING,
				    FILE_ATTRIBUTE_NORMAL | FILE_FLAG_NO_BUFFERING | FILE_FLAG_RANDOM_ACCESS, NULL );

	if( f != INVALID_HANDLE_VALUE )
	{
		// get file size
		DWORD hi;
		DWORD lo = GetFileSize(f, &hi);
		size = (__int64(hi) << 32) | (__int64(lo) & 0xffffffff);

		CloseHandle( f );
	}

	return size;
}


void CHDTVtoMPEG2Dlg::CreateTSIFileName( CString sName )
{
	int index;
	CString buff;
	int i = 1;

	index = sName.ReverseFind( '.' );
	sName = sName.Left( index );
	sName += ".tri";

	// TRI directory specified in ini file
	if( m_TRIDirectory[0] != '\0' )
	{
		index = sName.ReverseFind( '\\' );
		sName = sName.Right( sName.GetLength() -  index );

		CString sTemp = m_TRIDirectory;
		sTemp += sName;
		
		sName = sTemp;
	}
	 
	WritePrivateProfileString( "Settings", "TRIOut", m_TRIDirectory, inipath );
	
	// PathFileExists limited to 260 char filename
	// 50 max unique file names, then keep overwritting the last one
    while( PathFileExists((LPCTSTR)sName ) && i < 50 )
	{	
		if( i == 1 )
		{
			index = sName.ReverseFind( '.' );
			buff = sName.Left( index );
		}
		else
		{
			index = sName.ReverseFind( '-' );
			buff = sName.Left( index );
		}
	
		sName.Format("%s-%d.tri", buff, i );
		++i;
	}

	m_TRIFileName = sName;
}	

// open the TRI file
// the TRI file is only opened when the Include\Exclude\Move buttons
//  are selected.  This is so that .tri files aren't created when you 
//  just what to look at a TS file and not edit it.
void CHDTVtoMPEG2Dlg::OpenTRIFile()
{
	if( m_CreateTRIFiles )// ini flag set
	{
		// don't try to open the file if it's already open
		if( m_tsiFile.m_hFile == CFile::hFileNull )
		{
			if( !m_tsiFile.Open( m_TRIFileName, CFile::modeCreate | CFile::modeWrite, &e ) )
			{
				TCHAR error[1024];
				CString msg;

				e.GetErrorMessage( error, 1024 );

				msg.Format("Unable to create .TRI file!\n\n%s", error );
				MessageBox( msg );

				return;
			}
		}
	}
}

// process .tri file
bool CHDTVtoMPEG2Dlg::ProcessTRIFile( CString sPath )
{
	CStdioFile file;
	CFileException e;
	CString sLine;
	__int64 offset;
	CString sOffset;
	RegionType type;
	CString sType;
	int index;
	CString ext;

	if( !file.Open( sPath, CFile::modeRead ) )
	{
		MessageBox( "Unable to open .TRI file!" );
		return false;
	}


	// process file contents
	while( file.ReadString( sLine ) != NULL )
	{
		// file name
		if( sLine.Find( ":" ) != -1 || sLine.Find( "\\\\" ) != -1 )
		{
			// check file extension
			index = sLine.ReverseFind( '.' );
			ext = sLine.Right( sLine.GetLength() - index - 1 );

			if( ext == "ts"   || ext == "TS"   ||
				ext == "tp"   || ext == "TP"   ||
				ext == "trs"  || ext == "TRS"  ||
				ext == "trp"  || ext == "TRP"  ||
				ext == "adtv" || ext == "ADTV" ||
				ext == "m2t"  || ext == "M2T"  ||
				ext == "rec"  || ext == "REC"  ||
				ext == "mpg"  || ext == "MPG" )
			{
				if( GetFileByteSize( sLine ) == 0 )
				{
					CString msg;
					msg.Format( "Discarding 0 byte file:\n%s", sLine );
					MessageBox( msg );
					continue;
				}

				AddFile( sLine );
			}
			else
			{
				MessageBox( "Transport stream must have a valid extension!", "HDTVtoMPEG2 - Invalid file type!", MB_OK | MB_ICONERROR );
				return false;
			}
		}

		else // it's region data
		{
			// get region type
			int index = sLine.Find(" ");

			sType = sLine.Right( sLine.GetLength() - ( index + 1 ));

			if( sType == "include" )
				type = rt_include;
			else if( sType == "exclude" )
				type = rt_exclude;
			else if( sType == "cut" )
				type = rt_cut;
			else
			{
				MessageBox( "Unknown Region Type" );
				OnClear();
				return false;
			}

			// get offset
			index = sLine.Find( ',' );
			sOffset = sLine.Left( index );
			offset = _atoi64( sOffset );

			// add the region data
			m_InputList.AddRegion( type, offset );

		}
	}

	CString sName;
	if( m_InputFiles.GetCount() != 0 )
	{
		m_InputFiles.GetText( 0, sName );
		if( !sName.IsEmpty() )
			CreateTSIFileName( sName );
	}

	return true;
}

void CHDTVtoMPEG2Dlg::OnDelete() 
{
	CString temp;

	if( m_InputFiles.GetSelCount() > 0 )
	{
		for( int i = m_InputFiles.GetCount() - 1; i >= 0; i-- )
		{
			if( m_InputFiles.GetSel( i ) )
			{
				temp = m_InputFiles.GetFullFileName( i );

				m_InputList.Delete( temp );
				m_InputFiles.DeleteString( i );
			}
		}
#ifdef _DEBUG
		// check fileCount is the same
		if( m_InputFiles.GetCount() != m_InputList.Count() )
			__asm int 3;
#endif
	}

	if( m_InputFiles.GetCount() == 0 )
	{
		m_ChannelCombo.ResetContent();
		m_OutputFile.SetWindowText( "" );
		m_NumChannels = 0;
		m_InputList.Clear();

		OnClear();
	}

	SetVisibleStates();

	ConfigureRegionsUI();
	UpdateRegionsUI();
	EmptyScanBuffer();

	// force focus back to our dialog
	SetFocus();
}

void CHDTVtoMPEG2Dlg::OnSelchangeInputfiles() 
{
	int movingIndex = -1;
	int moveToIndex = -1;

	// bail if mouse click only, no drag was done
	if( !m_InputFiles.WasMouseDrag() )
	{
		SetVisibleStates();
		return;
	}

	// get our indexes
	moveToIndex = m_InputFiles.GetMoveToIndex();
	movingIndex = m_InputFiles.GetMovingIndex();

	// nothing has really been moved
	if( moveToIndex == movingIndex )
	{
		SetVisibleStates();
		return;
	}

	// last entry "moved" to "end" of list.  It's already there!
	if( moveToIndex == -1 && movingIndex == m_InputFiles.GetCount() - 1 )
	{
		SetVisibleStates();
		return;
	}

	// file is already at that location!
	if( movingIndex + 1 == moveToIndex )
	{
		SetVisibleStates();
		return;
	}

	// a file has been moved
	m_InputList.AdjustList( movingIndex, moveToIndex );

	// redraw the UI
	SetVisibleStates();
	EmptyScanBuffer();
	OnSeekBegin();
	OnSeekForward1();
	ConfigureRegionsUI();
}

void CHDTVtoMPEG2Dlg::OnClear() 
{
	m_InputFiles.ResetContent();
	m_ChannelCombo.ResetContent();
	m_PIDCombo.ResetContent();
	pPIDDeleteList->RemoveAll();
	m_OutputFile.SetWindowText("");
	m_NumChannels = 0;
	m_InputList.Clear();
	m_InputFiles.ClearPathList();
	

	SetVisibleStates();

	ConfigureRegionsUI();
	UpdateRegionsUI();
	EmptyScanBuffer();

	Stream_Header.buffer = NULL;
	Stream_Header.bytes = 0;

	if( PatPmtPackets.pPAT_Packet )
	{
		delete []PatPmtPackets.pPAT_Packet;
		PatPmtPackets.pPAT_Packet = NULL;
	}

    for( int i = 0; i < PatPmtPackets.numOfPMTPackets; i++ )
	{
		delete []PatPmtPackets.pPMT_Packet[i];
		PatPmtPackets.pPMT_Packet[i] = NULL;
	}

	PatPmtPackets.numOfPMTPackets = 0;

	if( m_tsiFile.m_hFile != INVALID_HANDLE_VALUE )
		m_tsiFile.Close();
}

void CHDTVtoMPEG2Dlg::OnChangeMaxFileSize() 
{
	UpdateData(TRUE);

	char buf[20];
	sprintf(buf, "%d", m_MaxFileSize);
	WritePrivateProfileString("Settings", "MaxFileSize", buf, inipath);
}


void CHDTVtoMPEG2Dlg::OnCheckProgressImage() 
{
	char *s = "0";
	if (m_pCheckProgressImage->GetCheck())
		s = "1";

	WritePrivateProfileString("Settings", "ShowProgressImage", s, inipath);
}

void CHDTVtoMPEG2Dlg::OnSelchangeChannel() 
{
	int ch = m_ChannelCombo.GetCurSel();
	if (ch >= 0 && ch < m_NumChannels)
	{
		char buf[32];

		sprintf(buf, "0x%X", (WORD)m_Channels[ch].videopid);
		m_VideoPIDStr = buf;

		sprintf(buf, "0x%X", (WORD)m_Channels[ch].audiopid);
		m_AudioPIDStr = buf;

		sprintf(buf, "%dx%d%c", m_Channels[ch].width, m_Channels[ch].height, m_Channels[ch].progressiveSequence ? 'p' : 'i');
		m_pTxtResolution->SetWindowText(buf);

		char *s;
		switch (m_Channels[ch].aspectRatio)
		{
		case 1: s = "1:1"; break;
		case 2: s = "4:3"; break;
		case 3: s = "16:9"; break;
		case 4: s = "2.21:1"; break;
		default: s = "?"; break;
		}
		m_pTxtAspectRatio->SetWindowText(s);

		switch (m_Channels[ch].frameRate)
		{
		case 1: s = "23.976"; break;
		case 2: s = "24"; break;
		case 3: s = "25"; break;
		case 4: s = "29.97"; break;
		case 5: s = "30"; break;
		case 6: s = "50"; break;
		case 7: s = "59.94"; break;
		case 8: s = "60"; break;
		default:
//			float frameRate = (float)frame_rate_value * (frame_rate_extension_n + 1)  (frame_rate_extension_d + 1)
			s = "?"; break;
		}
		m_pTxtFrameRate->SetWindowText(s);

//		makeSpeedStr(buf, (float)(m_Channels[ch].bitRate * 400), 'b');
		makeSpeedStr( buf, (float)(m_Channels[ch].bitRate ), 'b' );
		m_pTxtBitRate->SetWindowText(buf);

		UpdateRegionsUI();

		UpdateData(FALSE);
	}
}

void CHDTVtoMPEG2Dlg::OnSelchangeFiletype() 
{
	m_Filetype = (Filetype)m_FiletypeCombo.GetCurSel();

	if ((m_Filetype < ft_MIN) || (m_Filetype > ft_MAX))
		m_Filetype = ft_MIN;

	RemoveOutputFilenameExtension(m_ExtensionStr);

	char key[16];
	sprintf(key, "ExtType%d", m_Filetype);
	GetPrivateProfileString("Settings", key, default_extensions[m_Filetype], m_ExtensionStr, 255, inipath);

	AddOutputFilenameExtension(m_ExtensionStr);

	m_Extension.SetWindowText(m_ExtensionStr);

	sprintf(key, "%d", m_Filetype);
	WritePrivateProfileString("Settings", "FileType", key, inipath);

	UpdateData(FALSE);
}

void CHDTVtoMPEG2Dlg::ConfigureRegionsUI()
{
	int bFiles = FALSE;
	uint StreamRate = 0;
	int index = 0;

	if (m_InputList.Count() > 0)
	{
		bFiles = TRUE;

		index = m_ChannelCombo.GetCurSel();
		StreamRate = m_Channels[index].bitRate / 8;

		if( StreamRate == 0 )
			StreamRate = STREAMRATE;

		// total up time
		m_MaxRange = int(100 * m_InputList.TotalBytes() / StreamRate );

		// if we currently had a file selected, try to find it again,
		// and set m_CurPos to the same time offset into the file.
		if (m_CurrentFileFullName[0])
		{
			TSFileInfo *ti = m_InputList.Find(m_CurrentFileFullName);
			if (ti)
			{
				m_CurPos = int((100 * (ti->m_totalBytesPrior + m_CurrentFileOffset) + StreamRate-1) / StreamRate);
			}
			else
			{
				m_CurPos = 0;
				m_CurrentFileName = '\0';
				m_CurrentFileOffset = 0;
			}
		}
		else
		{
			m_CurPos = 0;
			m_CurrentFileName = '\0';
			m_CurrentFileOffset = 0;
		}
	}
	else
	{
		m_MaxRange = 1;
		m_CurPos = 0;
		m_CurrentFileFullName[0] = '\0';
		m_CurrentFileName = '\0';
		m_CurrentFileOffset = 0;
	}

	makeTimeStr2(m_TotalTimeText, float(m_MaxRange)*0.01f);
	makeSizeStr(m_TotalSizeText, m_InputList.TotalBytes());

	m_pTotalTime->SetWindowText(m_TotalTimeText);
	m_pTotalSize->SetWindowText(m_TotalSizeText);

	m_pBeginInclude->EnableWindow(bFiles);
	m_pBeginExclude->EnableWindow(bFiles);
	m_pCut->EnableWindow(bFiles);
	m_pRemoveRegion->EnableWindow(bFiles);
	m_pClearAllRegions->EnableWindow(bFiles);
	m_pScan->EnableWindow(bFiles);
	m_pSeekBegin->EnableWindow(bFiles);
	m_pSeekBack3->EnableWindow(bFiles);
	m_pSeekBack2->EnableWindow(bFiles);
	m_pSeekBack1->EnableWindow(bFiles);
	m_pSeekForward1->EnableWindow(bFiles);
	m_pSeekForward2->EnableWindow(bFiles);
	m_pSeekForward3->EnableWindow(bFiles);
	m_pSeekEnd->EnableWindow(bFiles);
	m_pRegionSlider->EnableWindow(bFiles);
	m_pBrightnessSlider->EnableWindow(bFiles);
	m_pContrastSlider->EnableWindow(bFiles);

	m_brightnessSlider.SetRange(-5, 5);
	m_brightnessSlider.SetPos( m_Brightness );
	m_brightnessSlider.SetLineSize(1);
	m_brightnessSlider.SetPageSize(1);

	m_contrastSlider.SetRange(1, 11);
	m_contrastSlider.SetPos( m_Contrast );
	m_contrastSlider.SetLineSize(1);
	m_contrastSlider.SetPageSize(1);

	m_TimeSlider.SetRange(0, m_MaxRange);
	m_TimeSlider.SetPos(0);
	m_TimeSlider.SetPos(m_CurPos);
	m_TimeSlider.SetLineSize(1);
	m_TimeSlider.SetPageSize(10*100);

	m_currentGOPVidPid = -1;
	m_currentGOPStart = 0;
	m_currentGOPEnd = -1;
	m_currentGOPEndValid = false;
}

void CHDTVtoMPEG2Dlg::SetUpScanBuffer(void)
{
	// figures out scan buffer info based on sector alignment stuff
	m_scanBufferSize = m_scanBufferSizeX - 3 * m_scanBufferAlignment;

	// reset buffer
	m_scanBufferOffsetBytes = -1;
	m_scanBufferUsedBytes = 0;
}

bool CHDTVtoMPEG2Dlg::FillScanBuffer(__int64 start, __int64 end)
{
	if (!m_scanBuffer || !m_scanBufferX ||
		(start > end) ||
		((end - start) > m_scanBufferSize))
		return false;

	if (start < 0)
		start = 0;

	if (start == end)
	{
		// oops, zero bytes asked for. harmless.
	}
	else if ((m_scanBufferUsedBytes == 0) ||
		(end < m_scanBufferOffsetBytes) ||
		(start > m_scanBufferOffsetBytes + m_scanBufferUsedBytes))
	{
		// make new buffer
		m_scanBufferUsedBytes = FillBuffer(m_scanBuffer, start, end - start);
		if (m_scanBufferUsedBytes <= 0)
		{
			m_scanBufferUsedBytes = 0;
			return false;
		}
		m_scanBufferOffsetBytes = start;
	}
	else
	{
		// do we need to load some bytes ahead of what's already there?
		__int64 bytes = m_scanBufferOffsetBytes - start;
		if (bytes > 0)
		{
			if (bytes > m_scanBufferSize)
				bytes = m_scanBufferSize;
			__int64 oldSize = min(m_scanBufferUsedBytes, m_scanBufferSize - bytes);
			// first, move what's already there along by size bytes
			if (oldSize > 0)
				memmove(&m_scanBuffer[bytes], m_scanBuffer, (int)oldSize);
			// now fill in size bytes at start of buffer
			if (FillBuffer(m_scanBuffer, start, bytes) != bytes)
			{
				m_scanBufferUsedBytes = 0;
				return false;
			}

			m_scanBufferOffsetBytes = start;
			m_scanBufferUsedBytes = bytes + oldSize;
		}

		// do we need to load some bytes after what's already there?
		bytes = end - (m_scanBufferOffsetBytes + m_scanBufferUsedBytes);
		if (bytes > 0)
		{
			__int64 shift = bytes + m_scanBufferUsedBytes - m_scanBufferSize;
			__int64 oldSize = m_scanBufferUsedBytes;
			if (shift > 0)
			{
				// gotta move back what's there
				oldSize -= shift;
				memmove(m_scanBuffer, &m_scanBuffer[shift], (int)oldSize);
				m_scanBufferOffsetBytes += shift;
			}
			// now fill in size bytes at end of buffer
			bytes = FillBuffer(&m_scanBuffer[oldSize], end - bytes, bytes);
			if (bytes <= 0)
			{
				return false;
			}

			m_scanBufferUsedBytes = oldSize + bytes;
		}
	}

	return true;
}

__int64 CHDTVtoMPEG2Dlg::FillBuffer(void *dest, __int64 start, __int64 bytes)
{
	// in order to maximise the responsiveness of this function so that the
	// UI doesn't chunk badly due to the Windows disk cache trying to be helpful
	// by loading more of the file than we want not to mention the fact that unless
	// a user has about 10gb of ram there's no chance of caching an hour or more of
	// transport stream files, we're directly using the OS file operations, with
	// the FILE_FLAG_NO_BUFFERING flag set, which bypasses the disk cache. 

	__int64 bufferStart = 0;

	TSFileInfo *baseTi = m_InputList.Find(start);
	if (baseTi)
	{
		// fill in buffer
		TSFileInfo *ti = baseTi;
		while (ti && (bufferStart < bytes))
		{
			// make sure current file is open
			if (ti->m_f == INVALID_HANDLE_VALUE)
				ti->m_f = CreateFile(ti->m_fileName, GENERIC_READ, FILE_SHARE_READ | FILE_SHARE_WRITE, NULL, OPEN_EXISTING,
								FILE_ATTRIBUTE_NORMAL | FILE_FLAG_NO_BUFFERING | FILE_FLAG_RANDOM_ACCESS, NULL);
			if (ti->m_f == INVALID_HANDLE_VALUE)
				return -1;	// oops, file open failure

			__int64 fileOffset = start - ti->m_totalBytesPrior;

			__int64 bytesToRead = bytes - bufferStart;
			if (bytesToRead > (ti->m_fileSize - fileOffset))
				bytesToRead = ti->m_fileSize - fileOffset;

			// no buffering has some restrictions: all file references must be in sector size
			// alignment, and so must the memory address we're copying into. m_scanBufferAlignment
			// is the "sector size" we're aligning sizes and addresses to. Once we read in the sectors
			// we want, then we copy the actual requested data to the buffer specified.

			// align down to prior sector size
			__int64 fileOffsetAligned = fileOffset & (~((__int64)m_scanBufferAlignment - 1));
			// align up to next sector size
			__int64 bytesToReadAligned = ((fileOffset + bytesToRead + (__int64)m_scanBufferAlignment - 1) & (~((__int64)m_scanBufferAlignment - 1))) - fileOffsetAligned;

			LONG hi = (LONG)(fileOffsetAligned >> 32);
			LONG lo = SetFilePointer(ti->m_f, (LONG)(fileOffsetAligned&0xffffffff), &hi, FILE_BEGIN);
			__int64 newOffset = (__int64(hi) << 32) | (__int64(lo) & 0xffffffff);
			if ((GetLastError() != NO_ERROR) || (newOffset != fileOffsetAligned))
				return bufferStart;

			DWORD r;
			char *destAligned = (char*)(((int)m_scanBufferX + m_scanBufferAlignment - 1) & (~((__int64)m_scanBufferAlignment - 1)));
			if (!ReadFile(ti->m_f, destAligned, (DWORD)bytesToReadAligned, &r, NULL))
				r = 0;
			r -= int(fileOffset - fileOffsetAligned);
			if (r > (DWORD)bytesToRead)
				r = (DWORD)bytesToRead;
			if (r > 0)
				memcpy((char*)dest + int(bufferStart), destAligned + int(fileOffset - fileOffsetAligned), r);
			else
				r = 0;

			bufferStart += __int64(r);

			if (r < (DWORD)bytesToRead)
				return bufferStart;

			start += bytesToRead;

			while (ti && (start >= (ti->m_totalBytesPrior + ti->m_fileSize)))
				ti = ti->m_next;
		}
	}

	return bufferStart;
}

void CHDTVtoMPEG2Dlg::EmptyScanBuffer(void)
{
	m_scanBufferOffsetBytes = -1;
	m_scanBufferUsedBytes = 0;
}

float g_brightness, g_contrast;
bool g_fBrightness, g_fContrast;
void CHDTVtoMPEG2Dlg::UpdateRegionsUI(int code, __int64 seekto)
{
	bool bReg = false;
	int brightness = m_brightnessSlider.GetPos();
	int contrast = m_contrastSlider.GetPos();
	g_fBrightness = (brightness != 0);
	g_fContrast = (contrast != 6);
	g_brightness = brightness * 0.25f;
	g_contrast = contrast / 6.0f;

	uint StreamRate = 0;
	int index = 0;

	index = m_ChannelCombo.GetCurSel();
	StreamRate = m_Channels[index].bitRate / 8;
	if( StreamRate == 0 )
		StreamRate = STREAMRATE;

	if (m_InputList.Count() > 0)
	{
		bReg = true;

		m_VideoPID = get_hex(m_VideoPIDStr);

		if (m_VideoPID != m_currentGOPVidPid)
		{
			// oops, wrong channel, reset
			m_currentGOPVidPid = -1;
			m_currentGOPStart = 0;
			m_currentGOPEnd = -1;
			m_currentGOPEndValid = false;
		}

		if ((code == 0) || (code == 2) || (m_currentGOPStart > m_currentGOPEnd))
		{
			__int64 totalOffset = 0;
			if (code == 0)
			{
				// move to position of slider
				m_CurPos = m_TimeSlider.GetPos();
				if (m_CurPos < 0) m_CurPos = 0;
				if (m_CurPos >= m_MaxRange) m_CurPos = m_MaxRange - 1;

				totalOffset = (__int64(m_CurPos) * StreamRate ) / 100;
			}
			if (code == 2)
			{
				totalOffset = seekto;
			}
			TSFileInfo *ti = m_InputList.Find(totalOffset);
			if (ti)
			{
				// grab current file information
				m_CurrentFileOffset = totalOffset - ti->m_totalBytesPrior;
				strcpy(m_CurrentFileFullName, ti->m_fileName);
				m_CurrentFileName = strrchr(m_CurrentFileFullName, '\\') + 1;

				// see if we're already looking at this GOP
				if ((totalOffset >= m_currentGOPStart) && (totalOffset < m_currentGOPEnd))
				{
					m_CurPos = int((100 * m_currentGOPStart + StreamRate -1) / StreamRate );
					m_TimeSlider.SetPos(m_CurPos);
				}
				else
				{
					// fill buf
					FillScanBuffer(totalOffset - m_scanRange, totalOffset + m_scanRange);
					// Find previous I-frame (within reasonable distance)
					__int64 iFrameBytes = scan_for_Iframe_back(m_scanBuffer, m_scanBufferUsedBytes, m_scanBufferOffsetBytes,
								&m_InputList, totalOffset, m_scanRange, m_VideoPID);

					// If can't...
					if (iFrameBytes < 0)
					{
						// special case to allow selecting very start of file
						if (totalOffset == 0)
							iFrameBytes = 0;
						else
						{
							// find next I-frame (within reasonable distance)
							iFrameBytes = scan_for_Iframe_forward(m_scanBuffer, m_scanBufferUsedBytes, m_scanBufferOffsetBytes,
									&m_InputList, totalOffset, m_scanRange, m_VideoPID);
						}
					}

					if (iFrameBytes >= 0)
					{
						m_currentGOPVidPid = m_VideoPID;
						m_currentGOPStart = iFrameBytes;

						// decode I-frame into a non-m_activeBuffer
						pdlg_buffer *target = &m_pdlgBuffers[0];
						if (target == m_activeBuffer)
							target = &m_pdlgBuffers[1];

						// fill buf
						FillScanBuffer(m_currentGOPStart, m_currentGOPStart + m_scanRange);
						// find end of GOP (next I-frame) (within reasonable distance)
						iFrameBytes = scan_for_Iframe_forward(m_scanBuffer, m_scanBufferUsedBytes, m_scanBufferOffsetBytes,
									&m_InputList, m_currentGOPStart, m_scanRange, m_VideoPID);

						// now that m_scanBuffer is (probably) filled with data, grab I-frame...
						int bufOffset = int(m_currentGOPStart - m_scanBufferOffsetBytes);
						if (decode_I_frame(&m_scanBuffer[bufOffset],
									(int)m_scanBufferUsedBytes - bufOffset, m_VideoPID, target))
						{
// commented out because it caused a crash on Terry Petersons SD stream sample.
// also this appears to only be here for debug purposes.
//float scanRatio = 1.0f/16.0f;
//float e = compareFrameEdges(target, m_activeBuffer, scanRatio);
//_Error("%08x%08x %.5f\n", (uint)(m_currentGOPStart>>32), (uint)(m_currentGOPStart&0xffffffff), e);
							m_activeBuffer = target;
						}
						else
							m_activeBuffer = NULL;

						if (iFrameBytes >= 0)
						{
							m_currentGOPEnd = iFrameBytes;
							m_currentGOPEndValid = true;
						}
						else
						{
							// use closer of (+ reasonable distance) or (end of files)
							m_currentGOPEnd = m_currentGOPStart + m_scanBufferSize;
							if (m_currentGOPEnd > m_InputList.TotalBytes())
								m_currentGOPEnd = m_InputList.TotalBytes();
							m_currentGOPEndValid = false;
						}

						m_CurPos = int((100 * m_currentGOPStart + StreamRate-1) / StreamRate);
						m_TimeSlider.SetPos(m_CurPos);
					}
					else
					{
						m_currentGOPVidPid = -1;
						m_currentGOPStart = 0;
						m_currentGOPEnd = -1;
						m_currentGOPEndValid = false;
						m_activeBuffer = NULL;
					}
				}
			}
			else
			{
				m_currentGOPVidPid = -1;
				m_currentGOPStart = 0;
				m_currentGOPEnd = -1;
				m_currentGOPEndValid = false;
				m_activeBuffer = NULL;
			}
		}
		else
		{
			// increment or decrement by one I-frame
			bool ok = false;
			if (code < 0)
			{
				if (m_currentGOPStart > 0)
				{
					// fill buf
					FillScanBuffer(m_currentGOPStart - m_scanRange, m_currentGOPStart + m_scanRange);
					// find previous I-frame (within reasonable distance)
					__int64 iFrameBytes = scan_for_Iframe_back(m_scanBuffer, m_scanBufferUsedBytes, m_scanBufferOffsetBytes,
								&m_InputList, m_currentGOPStart - 1, m_scanRange, m_VideoPID);

					if ((iFrameBytes < 0) && (m_currentGOPStart < (__int64)m_scanBufferSize))
						// special case to allow selecting start of file
						iFrameBytes = 0;

					if (iFrameBytes >= 0)
					{
						m_currentGOPVidPid = m_VideoPID;
						m_currentGOPStart = iFrameBytes;

						ok = true;
					}
				}
			}
			else
			{
				if(m_currentGOPEndValid)
				{
					m_currentGOPVidPid = m_VideoPID;
					m_currentGOPStart = m_currentGOPEnd;

					ok = true;
				}
			}

			if (ok)
			{
				// fill buf ahead + scan_distance
				FillScanBuffer(m_currentGOPStart, m_currentGOPStart + m_scanRange);
				// find next I-frame (within reasonable distance)
				__int64 iFrameBytes = scan_for_Iframe_forward(m_scanBuffer, m_scanBufferUsedBytes, m_scanBufferOffsetBytes,
							&m_InputList, m_currentGOPStart, m_scanRange, m_VideoPID);

				// now that m_scanBuffer is (probably) filled with data, grab I-frame...
				// decode I-frame into a non-m_activeBuffer
				pdlg_buffer *target = &m_pdlgBuffers[0];
				if (target == m_activeBuffer)
					target = &m_pdlgBuffers[1];
				int bufOffset = int(m_currentGOPStart - m_scanBufferOffsetBytes);
				if (decode_I_frame(&m_scanBuffer[bufOffset],
							(int)m_scanBufferUsedBytes - bufOffset, m_VideoPID, target))
				{
//float scanRatio = 1.0f/16.0f;
//float e = compareFrameEdges(target, m_activeBuffer, scanRatio);
//_Error("%08x%08x %.5f\n", (uint)(m_currentGOPStart>>32), (uint)(m_currentGOPStart&0xffffffff), e);
					m_activeBuffer = target;
				}
				else
					m_activeBuffer = NULL;

				if (iFrameBytes >= 0)
				{
					m_currentGOPEnd = iFrameBytes;
					m_currentGOPEndValid = true;
				}
				else
				{
					// use closer of (+ reasonable distance) or (end of files)
					m_currentGOPEnd = m_currentGOPStart + m_scanBufferSize;
					if (m_currentGOPEnd > m_InputList.TotalBytes())
						m_currentGOPEnd = m_InputList.TotalBytes();
					m_currentGOPEndValid = false;
				}

				TSFileInfo *ti = m_InputList.Find(m_currentGOPStart);
				if (ti)
				{
					// grab current file name
					strcpy(m_CurrentFileFullName, ti->m_fileName);
					m_CurrentFileName = strrchr(m_CurrentFileFullName, '\\') + 1;
				}
			}

			// set the new slider position
			m_CurPos = int((100 * m_currentGOPStart + StreamRate-1) / StreamRate);
			m_TimeSlider.SetPos(m_CurPos);
		}
	}
	else
	{
		m_currentGOPVidPid = -1;
		m_currentGOPStart = 0;
		m_currentGOPEnd = -1;
		m_currentGOPEndValid = false;
		m_activeBuffer = NULL;
		m_CurrentFileFullName[0] = '\0';
		m_CurrentFileName = '\0';
	}

	m_InputList.CloseFilesOutsideRange(m_scanBufferOffsetBytes, m_scanBufferOffsetBytes + m_scanBufferUsedBytes);

	int bPrev = FALSE;
	int bNext = FALSE;
	int bMove = FALSE;

	if (m_currentGOPStart <= m_currentGOPEnd)
	{
		makeTimeStr2(m_CurrentTimeText, float(m_currentGOPStart)*(1.0f/float(StreamRate)));
		makeSizeStr(m_CurrentSizeText, m_currentGOPStart);

//		if (m_currentGOPStart > 0)
		if (m_currentGOPStart >= 0)
			bPrev = TRUE;

		TSRegionInfo *tri = m_InputList.FindRegion(m_currentGOPStart);
		if (m_InputList.NextRegion(tri))
			bNext = TRUE;

		if (tri)
		{
			__int64 triTotalOffset = tri->m_offset + tri->m_parent->m_totalBytesPrior;
			if (triTotalOffset == m_currentGOPStart)
				bMove = TRUE;
		}
	}
	else
	{
		makeTimeStr2(m_CurrentTimeText, float(m_CurPos)*0.01f);
		makeSizeStr(m_CurrentSizeText, (__int64(m_CurPos) * StreamRate ) / 100);

		m_CurPos = m_TimeSlider.GetPos();
		if (m_CurPos < 0) m_CurPos = 0;
		if (m_CurPos >= m_MaxRange) m_CurPos = m_MaxRange - 1;
		__int64 totalOffset = (__int64(m_CurPos) * StreamRate) / 100;

		if (totalOffset > 0)
			bPrev = TRUE;

		TSRegionInfo *tri = m_InputList.FindRegion(totalOffset);
		if (m_InputList.NextRegion(tri))
			bNext = TRUE;
	}

	makeTimeStr2(m_IncludedTimeText, float(m_InputList.TotalIncludedBytes())*(1.0f/float(StreamRate)));
	makeSizeStr(m_IncludedSizeText, m_InputList.TotalIncludedBytes());

	m_pCurrentTime->SetWindowText(m_CurrentTimeText);
	m_pCurrentTime->SendMessage(WM_PAINT);

	m_pCurrentSize->SetWindowText(m_CurrentSizeText);
	m_pCurrentSize->SendMessage(WM_PAINT);

	m_pIncludedTime->SetWindowText(m_IncludedTimeText);
	m_pIncludedTime->SendMessage(WM_PAINT);

	m_pIncludedSize->SetWindowText(m_IncludedSizeText);
	m_pIncludedSize->SendMessage(WM_PAINT);

	m_pCurrentFile->SetWindowText(m_CurrentFileName);
	m_pCurrentFile->SendMessage(WM_PAINT);

	m_pPreviewFrame->ShowWindow(m_activeBuffer == NULL);
	m_pRegionFrame->ShowWindow(!bReg);

	m_pPrevRegion->EnableWindow(bPrev);
	m_pNextRegion->EnableWindow(bNext);
	m_pMoveRegionBack->EnableWindow(bMove);
	m_pMoveRegionForward->EnableWindow(bMove);

	// tell image to redraw?
	if (bPrev)
		DrawPreview();

	// tell regions graph thingy to draw?
	if (bReg)
		DrawRegions();

	UpdateData(FALSE);
}

void CHDTVtoMPEG2Dlg::OnHScroll(UINT nSBCode, UINT nPos, CScrollBar *pScrollBar)
{
	int nControl = pScrollBar->GetDlgCtrlID();
	CSliderCtrl *pControl = (CSliderCtrl*)GetDlgItem( nControl );

	switch( nControl )
	{
		case IDC_CONTRAST_SLIDER:
			ASSERT( pControl != NULL );
			m_Contrast = pControl->GetPos();
			break;

		case IDC_BRIGHTNESS_SLIDER:
			ASSERT( pControl != NULL );
			m_Brightness = pControl->GetPos();
			break;
	}

	int code = 0;
	switch( nSBCode )
	{
		case SB_LINELEFT:
			// jump 1 I-frame back
			code = -1;
			break;

		case SB_LINERIGHT:
			// jump 1 I-frame ahead
			code = 1;
			break;
	}

	UpdateRegionsUI( code );
}

void CHDTVtoMPEG2Dlg::OnConvert() 
{
	UpdateData(TRUE);

	m_VideoPID = get_hex(m_VideoPIDStr);
	m_AudioPID = get_hex(m_AudioPIDStr);

	// don't allow conversion without a channel selected
	bool bAnySelected = false;
	for( int i = 0; i < m_ChannelCombo.GetCount(); i++ )
	{
		if( m_ChannelCombo.GetCheck( i ) )
		{
			bAnySelected = true;
			break;
		}
	}
	if( !bAnySelected )
	{
		MessageBox("Please check a Channel to Process!", "Warning!", MB_ICONWARNING | MB_OK );
		return;
	}

	m_OutputFile.GetWindowText(m_OutputFileName, 256);

	SaveOutputFilePath(m_OutputFileName);

	if (m_InputList.Find(m_OutputFileName))
	{
		MessageBox("Input file and Output file can not be the same!", "Error!", MB_ICONERROR | MB_OK );
		return;
	}

	SaveRegionInfo();

	m_bCancel = false;

	// make buffers unavailable until progress dialog says they're OK
	pdlg_buffers[0].rendered = 1;
	pdlg_buffers[1].rendered = 1;

	m_ShowPreview = false;
	UINT nIDTemplate = IDD_PROGRESS;
	if (m_pCheckProgressImage->GetCheck())
	{
		nIDTemplate = IDD_PROGRESS_WITH_PREVIEW;
		m_ShowPreview = true;
	}

	CProgressDialog pdlg(nIDTemplate);
	m_ProgressDlg = &pdlg;

	if (nIDTemplate == IDD_PROGRESS_WITH_PREVIEW)
		m_ProgressDlg->SetInfo(pdlg_buffers, &m_InputList);
	else
		m_ProgressDlg->SetInfo(NULL, &m_InputList);

	unsigned int threadID;
	HANDLE hThread = (HANDLE)_beginthreadex(NULL, 1024 * 16, StaticConvertThread, this, 0, &threadID);
	CloseHandle(hThread);

	m_ProgressDlg->DoModal();

	m_ProgressDlg = NULL;

	m_bCancel = true;
}

unsigned int WINAPI CHDTVtoMPEG2Dlg::StaticConvertThread(LPVOID data)
{
//	SetThreadPriority(GetCurrentThread(), THREAD_PRIORITY_ABOVE_NORMAL);

	CHDTVtoMPEG2Dlg* dlg = (CHDTVtoMPEG2Dlg*)data;
	dlg->ConvertThread();

	return 0;
}

void CHDTVtoMPEG2Dlg::ConvertThread()
{
	if (m_ProgressDlg)
	{
		time((time_t*)&m_ProgressDlg->m_lTimeStart);
		m_ProgressDlg->m_fTimeTaken = 0.0f;
	}

	// start filereadbuffer up
	FileReadBuffer readBuffer;
	readBuffer.Start(4*1024*1024, m_scanBufferAlignment);

	switch (m_Filetype)
	{
	case ft_mpg:
		convert_files_to_mpg(&m_InputList, m_OutputFileName,
			m_VideoPID, m_AudioPID, m_MaxFileSize,
			&readBuffer, StaticConvertCallback, this,
			(m_ProgressDlg==NULL)?NULL:m_ProgressDlg->m_hWnd, m_pPidInfoList );
		break;

	case ft_ts:
	case ft_HiDTV:
		int vertical_size_value = 0;
		int index = 0;
		index = m_ChannelCombo.GetCurSel();
		vertical_size_value = m_Channels[index].height;

		convert_files_to_ts( &m_InputList,
							 m_OutputFileName,
							 m_Filetype,
							 m_VideoPID,
							 m_AudioPID,
							 m_MaxFileSize,
							 &readBuffer,
							 StaticConvertCallback,
							 this,
						     (m_ProgressDlg==NULL)?NULL:m_ProgressDlg->m_hWnd,
							 m_ShowPreview,
							 m_pPidInfoList,
							 pHiddenPidDeleteList,
							 m_Clips,
							 m_PreserveBitRate,
							 vertical_size_value,
							 m_SetVerticalSizeTo1080 );
		break;
	}

	if (m_ProgressDlg)
	{
		m_ProgressDlg->m_Finished = true;
	}

	readBuffer.Stop();
}

bool CHDTVtoMPEG2Dlg::StaticConvertCallback(char *inputFileName, char *outputFileName,
	__int64 curTotalPos, __int64 curTotalIn, __int64 totalIn,
	__int64 curTotalOut, long timeDiscrepancy, void* data)
{
	CHDTVtoMPEG2Dlg* dlg = (CHDTVtoMPEG2Dlg*)data;
	return dlg->ConvertCallback(inputFileName, outputFileName, curTotalPos, curTotalIn, totalIn, curTotalOut, timeDiscrepancy);
}

bool CHDTVtoMPEG2Dlg::ConvertCallback(char* inputFileName, char* outputFileName,
	__int64 curTotalPos, __int64 curTotalIn, __int64 totalIn,
	__int64 curTotalOut, long timeDiscrepancy)
{
	if (m_bCancel)
		return false;

	if (m_ProgressDlg)
	{
		char *p = strrchr(inputFileName, '\\');
		if (!p || !(*(++p)))
			p = inputFileName;
		strcpy(m_ProgressDlg->m_InputFileName, p);

		if( outputFileName )
		{
			if( m_ShowPreview == false )
				m_ProgressDlg->m_pOutputFileText->ShowWindow( true );

			p = strrchr(outputFileName, '\\');
			if (!p || !(*(++p)))
				p = outputFileName;

			strcpy(m_ProgressDlg->m_OutputFileName, p);
		}
		else
		{
			// don't display "Output File:" when doing commercial scan
			m_ProgressDlg->m_pOutputFileText->ShowWindow( false );

			p = "Scanning file...";
			strcpy(m_ProgressDlg->m_OutputFileName, p);

			// don't display "To go:" when doing commercial scan
			m_ProgressDlg->m_pToGoText->ShowWindow( false );
			m_ProgressDlg->m_pTimeToGoText->ShowWindow( false );
		}

		m_ProgressDlg->m_ProgressPos = curTotalPos;

		__int64 progress = curTotalIn * (__int64)1000 / totalIn;
		m_ProgressDlg->m_ProgressTotalPos = (int)progress;

		sprintf(m_ProgressDlg->m_Title, "%d%% Progress", (int)(progress / 10));

		// time & speed calculations
		if (timeDiscrepancy > 0)
			m_ProgressDlg->m_lTimeStart += timeDiscrepancy;

//		long tmp;
		time_t tmp;
		time( &tmp );
		float t = (float)(tmp - m_ProgressDlg->m_lTimeStart);
		if (t >= (m_ProgressDlg->m_fTimeTaken + 0.999f))
		{
			m_ProgressDlg->m_fTimeTaken = t;

			char buf[64];
			makeTimeStr(buf, m_ProgressDlg->m_fTimeTaken);
			buf[15] = '\0';
			strcpy(m_ProgressDlg->m_TimeTakenText, buf);

			t = curTotalIn / m_ProgressDlg->m_fTimeTaken;
			makeSpeedStr(buf, t, 'B');
			buf[15] = '\0';

			// don't display the conversion speed when commercial scanning
			if( outputFileName )
				strcpy(m_ProgressDlg->m_ConversionSpeedInText, buf);

			if (t > 0.0f)
			{
				t = (totalIn - curTotalIn) / t;
				makeTimeStr(buf, t);
				buf[15] = '\0';
				strcpy(m_ProgressDlg->m_TimeToGoText, buf);
			} else
				m_ProgressDlg->m_TimeToGoText[0] = '\0';

			if (curTotalOut >= 0)
			{
				t = curTotalOut / m_ProgressDlg->m_fTimeTaken;
				makeSpeedStr(buf, t, 'B');
				buf[15] = '\0';
			}
			else
				buf[0] = '\0';

			strcpy(m_ProgressDlg->m_ConversionSpeedOutText, buf);
		}
	}

	return true;
}

void CHDTVtoMPEG2Dlg::OnChangeExtension()
{
	CString name;

	RemoveOutputFilenameExtension(m_ExtensionStr);

	m_Extension.GetWindowText(name);

	strcpy(m_ExtensionStr, name);

	char key[16];
	sprintf(key, "ExtType%d", m_Filetype);
	WritePrivateProfileString("Settings", key, m_ExtensionStr, inipath);

	AddOutputFilenameExtension(m_ExtensionStr);
}

void CHDTVtoMPEG2Dlg::OnBeginExclude()
{
	if ((m_InputList.Count() > 0) && (m_currentGOPStart <= m_currentGOPEnd))
	{
		if (m_InputList.AddRegion(rt_exclude, m_currentGOPStart))
		{
			DrawRegions();
			UpdateRegionsUI(2, m_currentGOPStart);
		}
	}
	m_TimeSlider.SetFocus();

	SaveRegionInfo();
	OpenTRIFile();
}

void CHDTVtoMPEG2Dlg::OnBeginInclude()
{
	if ((m_InputList.Count() > 0) && (m_currentGOPStart <= m_currentGOPEnd))
	{
		if (m_InputList.AddRegion(rt_include, m_currentGOPStart))
		{
			DrawRegions();
			UpdateRegionsUI(2, m_currentGOPStart);
		}
	}
	m_TimeSlider.SetFocus();

	SaveRegionInfo();
	OpenTRIFile();
}

void CHDTVtoMPEG2Dlg::OnBeginCut()
{
	if(( m_InputList.Count() > 0 ) && ( m_currentGOPStart <= m_currentGOPEnd ))
	{
		if( m_InputList.AddRegion( rt_cut, m_currentGOPStart ))
		{
			DrawRegions();
			UpdateRegionsUI( 2, m_currentGOPStart );
		}
	}

	m_TimeSlider.SetFocus();

	SaveRegionInfo();
	OpenTRIFile();
}

void CHDTVtoMPEG2Dlg::OnRemoveRegion()
{
	uint StreamRate = 0;
	int index = 0;

	index = m_ChannelCombo.GetCurSel();
	StreamRate = m_Channels[index].bitRate / 8;

	RedrawWindow();

	if (m_InputList.Count() > 0)
	{
		m_CurPos = m_TimeSlider.GetPos();
		if (m_CurPos < 0) m_CurPos = 0;
		if (m_CurPos >= m_MaxRange) m_CurPos = m_MaxRange - 1;

		__int64 totalOffset = (__int64(m_CurPos) * StreamRate ) / 100;

		TSRegionInfo *tri = m_InputList.FindRegion(totalOffset);
		if (tri)
		{
			m_InputList.DeleteRegion(tri);
			DrawRegions();
			UpdateRegionsUI(2, totalOffset);
		}
	}
	m_TimeSlider.SetFocus();

	SaveRegionInfo();
	OpenTRIFile();
}

void CHDTVtoMPEG2Dlg::OnClearAllRegions()
{
	// give the use a RUSure dialog just in case they hit the 
	//  Remove All button by accident
	CString msg;
	msg.Format("Are you SURE you want to delete\nALL of the Region markers?");
	if( AfxMessageBox(msg, MB_YESNO | MB_ICONEXCLAMATION ) == IDNO )
		return;

	m_InputList.ClearRegions();

	m_InputList.ClearPrevIFrameOffsets();

	RedrawWindow();
	UpdateRegionsUI();

	m_TimeSlider.SetFocus();

	// close and preserve the current region information
	if( m_tsiFile.m_hFile != INVALID_HANDLE_VALUE )
		m_tsiFile.Close();
}

void CHDTVtoMPEG2Dlg::OnPeekAtFile()
{
	if (m_InputList.Count() > 0)
		peek_at_file(m_InputList.GetHead()->m_fileName, NULL, this);
}

void CHDTVtoMPEG2Dlg::OnSeekBegin()
{
	UpdateRegionsUI(2, 0);
	m_TimeSlider.SetFocus();
}

void CHDTVtoMPEG2Dlg::OnSeekEnd()
{
	UpdateRegionsUI(2, m_InputList.TotalBytes());
	m_TimeSlider.SetFocus();
}

void CHDTVtoMPEG2Dlg::OnSeekBack3()
{
	int newPos = m_TimeSlider.GetPos() - 60*100;
	if (newPos < 0)
		newPos = 0;
	m_TimeSlider.SetPos(newPos);
	UpdateRegionsUI(0);
	m_TimeSlider.SetFocus();
}

void CHDTVtoMPEG2Dlg::OnSeekForward3()
{
	int newPos = m_TimeSlider.GetPos() + 60*100;
	if (newPos > m_MaxRange)
		newPos = m_MaxRange;
	m_TimeSlider.SetPos(newPos);
	UpdateRegionsUI(0);
	m_TimeSlider.SetFocus();
}

void CHDTVtoMPEG2Dlg::OnSeekBack2()
{
	int newPos = m_TimeSlider.GetPos() - 10*100;
	if (newPos < 0)
		newPos = 0;
	m_TimeSlider.SetPos(newPos);
	UpdateRegionsUI(0);
	m_TimeSlider.SetFocus();
}

void CHDTVtoMPEG2Dlg::OnSeekForward2()
{
	int newPos = m_TimeSlider.GetPos() + 10*100;
	if (newPos > m_MaxRange)
		newPos = m_MaxRange;
	m_TimeSlider.SetPos(newPos);
	UpdateRegionsUI(0);
	m_TimeSlider.SetFocus();
}

void CHDTVtoMPEG2Dlg::OnSeekBack1()
{
	UpdateRegionsUI(-1);
	m_TimeSlider.SetFocus();
}

void CHDTVtoMPEG2Dlg::OnSeekForward1()
{
	UpdateRegionsUI(1);
	m_TimeSlider.SetFocus();
}

BOOL CHDTVtoMPEG2Dlg::OnMouseWheel( UINT nFlags, short zDelta, CPoint pt )
{
	if( zDelta < 0 )
		OnSeekForward1();

	if( zDelta > 0 )
		OnSeekBack1();

	return true;
}

void CHDTVtoMPEG2Dlg::OnLButtonDown(UINT nFlags, CPoint point)
{
	if ((point.x >= m_previewRect.left) && (point.x < m_previewRect.right) &&
		(point.y >= m_previewRect.top) && (point.y < m_previewRect.bottom) &&
		(m_InputList.Count() > 0))
	{
		// clicked in preview rectangle
		m_TimeSlider.SetFocus();
	}
	else if ((point.x >= m_regionRect.left) && (point.x < m_regionRect.right) &&
			(point.y >= m_regionRect.top) && (point.y < m_regionRect.bottom) &&
			(m_InputList.Count() > 0))
	{
		// clicked in region rectangle
		__int64 totalOffset = m_InputList.TotalBytes() * (__int64)(point.x - m_regionRect.left) / (__int64)m_regionRect.Width();

		TSRegionInfo *tri = m_InputList.FindRegion(totalOffset);
		if (tri)
			totalOffset = tri->m_offset + tri->m_parent->m_totalBytesPrior;
		else
			totalOffset = 0;

		UpdateRegionsUI(2, totalOffset);
		m_TimeSlider.SetFocus();
	}
	else
		CDialog::OnLButtonDown(nFlags, point);
}

void CHDTVtoMPEG2Dlg::OnPrevRegion()
{
	uint StreamRate = 0;
	int index = 0;

	index = m_ChannelCombo.GetCurSel();
	StreamRate = m_Channels[index].bitRate / 8;
	__int64 totalOffset = m_currentGOPStart;
	if (m_currentGOPStart > m_currentGOPEnd)
	{
		m_CurPos = m_TimeSlider.GetPos();
		if (m_CurPos < 0) m_CurPos = 0;
		if (m_CurPos >= m_MaxRange) m_CurPos = m_MaxRange - 1;
		totalOffset = (__int64(m_CurPos) * StreamRate) / 100;
	}

	TSRegionInfo *tri = m_InputList.FindRegion(totalOffset);
	if (tri && (totalOffset == tri->m_offset + tri->m_parent->m_totalBytesPrior))
		tri = m_InputList.PrevRegion(tri);
	if (tri)
		totalOffset = tri->m_offset + tri->m_parent->m_totalBytesPrior;
	else
		totalOffset = 0;

	UpdateRegionsUI(2, totalOffset);
	m_TimeSlider.SetFocus();
}

void CHDTVtoMPEG2Dlg::OnNextRegion()
{
	uint StreamRate = 0;
	int index = 0;

	index = m_ChannelCombo.GetCurSel();
	StreamRate = m_Channels[index].bitRate / 8;
	__int64 totalOffset = m_currentGOPStart;
	if (m_currentGOPStart > m_currentGOPEnd)
	{
		m_CurPos = m_TimeSlider.GetPos();
		if (m_CurPos < 0) m_CurPos = 0;
		if (m_CurPos >= m_MaxRange) m_CurPos = m_MaxRange - 1;
		totalOffset = (__int64(m_CurPos) * StreamRate ) / 100;
	}

	TSRegionInfo *tri = m_InputList.NextRegion(m_InputList.FindRegion(totalOffset));
	if (tri)
	{
		totalOffset = tri->m_offset + tri->m_parent->m_totalBytesPrior;

		UpdateRegionsUI(2, totalOffset);
	}
	m_TimeSlider.SetFocus();
}

void CHDTVtoMPEG2Dlg::OnMoveRegionBack()
{
	__int64 oldCurrentGOPStart = m_currentGOPStart;
	TSRegionInfo *tri = m_InputList.FindRegion(m_currentGOPStart);
	if (tri)
	{
		UpdateRegionsUI(-1);

		if ((m_currentGOPStart <= m_currentGOPEnd) && (m_currentGOPStart < oldCurrentGOPStart))
		{
			RegionType rt = tri->m_type;
			m_InputList.DeleteRegion(tri);
			tri = m_InputList.AddRegion(rt, m_currentGOPStart);
			UpdateRegionsUI(2, m_currentGOPStart);
		}
	}
	m_TimeSlider.SetFocus();

	SaveRegionInfo();

	OpenTRIFile();
}

void CHDTVtoMPEG2Dlg::OnMoveRegionForward()
{
	__int64 oldCurrentGOPStart = m_currentGOPStart;
	TSRegionInfo *tri = m_InputList.FindRegion(m_currentGOPStart);
	if (tri)
	{
		UpdateRegionsUI(1);

		if ((m_currentGOPStart <= m_currentGOPEnd) && (m_currentGOPStart > oldCurrentGOPStart))
		{
			RegionType rt = tri->m_type;
			m_InputList.DeleteRegion(tri);
			tri = m_InputList.AddRegion(rt, m_currentGOPStart);
			UpdateRegionsUI(2, m_currentGOPStart);
		}
	}
	m_TimeSlider.SetFocus();

	SaveRegionInfo();

	OpenTRIFile();
}

void CHDTVtoMPEG2Dlg::SaveRegionInfo()
{
	TSFileInfo *tsi;
	TSRegionInfo *tri;
	__int64 offset;
	RegionType type;
	CString fileName;
	CString sType;
	CString sCurrentFileFullName;
	CString sTRI_Filename;
	CString string;
	CFile file;
	CFileException e;

	if( m_tsiFile.m_hFile == INVALID_HANDLE_VALUE )
		return;

	m_tsiFile.SetLength( 0 );

	tsi = m_InputList.GetHead();

	while( tsi )
	{
		fileName = tsi->m_fileName;
		fileName += "\n";

		m_tsiFile.Write( fileName, fileName.GetLength() );

		tri = tsi->m_head;

		while( tri )
		{
			offset = tri->m_offset;
			offset += tsi->m_totalBytesPrior;

			type = tri->m_type;

			if( type == 0 )
				sType = "include";
			else if( type == 1 )
				sType = "exclude";
			else
				sType = "cut";

			string.Format( "%I64d, ", offset );
			m_tsiFile.Write( string, string.GetLength() );
			string.Format( "%s\n", sType );
			m_tsiFile.Write( string, string.GetLength() );

			tri = tri->m_next;
		}

		tsi = tsi->m_next;
	}
}

void CHDTVtoMPEG2Dlg::OnScan()
{
	int index = m_ChannelCombo.GetCurSel();
	if( m_Channels[index].width < 1280 )
	{
		MessageBox("Commercial Scan only works on HDTV Channels with\n16x9 programming and 4x3 \"black bar\" commercials.", "SD Channels not supported!" );
		return;
	}

	UpdateData(TRUE);

	m_VideoPID = get_hex(m_VideoPIDStr);

	if (m_InputList.Count() <= 0)
	{
		MessageBox("No files selected to scan!", "Error");
		return;
	}

	m_bCancel = false;

	m_ShowPreview = false;
	UINT nIDTemplate = IDD_PROGRESS;
//	if (m_pCheckProgressImage->GetCheck())
//	{
//		nIDTemplate = IDD_PROGRESS_WITH_PREVIEW;
//		m_ShowPreview = true;
//	}

	CProgressDialog pdlg(nIDTemplate);
	m_ProgressDlg = &pdlg;

	m_ProgressDlg->SetInfo(pdlg_buffers, &m_InputList);

	unsigned int threadID;
	HANDLE hThread = (HANDLE)_beginthreadex(NULL, 1024 * 16, StaticScanThread, this, 0, &threadID);
	CloseHandle(hThread);

	m_ProgressDlg->DoModal();

	m_ProgressDlg = NULL;

	m_bCancel = true;

	UpdateRegionsUI();
}


unsigned int WINAPI CHDTVtoMPEG2Dlg::StaticScanThread(LPVOID data)
{
//	SetThreadPriority(GetCurrentThread(), THREAD_PRIORITY_ABOVE_NORMAL);

	CHDTVtoMPEG2Dlg* dlg = (CHDTVtoMPEG2Dlg*)data;
	dlg->ScanThread();

	return 0;
}

void CHDTVtoMPEG2Dlg::ScanThread()
{
	if (m_ProgressDlg)
	{
		time((time_t*)&m_ProgressDlg->m_lTimeStart);
		m_ProgressDlg->m_fTimeTaken = 0.0f;
	}

	// start filereadbuffer up
	FileReadBuffer readBuffer;
	readBuffer.Start(4*1024*1024, m_scanBufferAlignment);

	// call subthread
	__int64 minExcludeSize = 30*2500000;	// 30 secs
	__int64 responseSize = 4*2500000;	// 4 secs
	float maxError = 0.10f;
	float scanRatio = 1.0f/16.0f;

/* old scan call
	scan_for_excludes(&m_InputList, m_VideoPID,
		minExcludeSize, responseSize, maxError, scanRatio,
		&readBuffer, StaticConvertCallback, this,
		(m_ProgressDlg==NULL)?NULL:m_ProgressDlg->m_hWnd);
*/

	uint streamrate = 0;
	int index = 0;

	index = m_ChannelCombo.GetCurSel();
	streamrate = m_Channels[index].bitRate / 8;

	scan_for_commercials( &m_InputList,
						  m_VideoPID,
						  minExcludeSize,
						  responseSize,
						  maxError,
						  scanRatio,
						  &readBuffer,
						  StaticConvertCallback,
						  this,
					      (m_ProgressDlg==NULL)?NULL:m_ProgressDlg->m_hWnd,
						  streamrate,
						  m_MaxSeekSize,
						  m_IncludeBlackFrames );

	if (m_ProgressDlg)
	{
		m_ProgressDlg->m_Finished = true;
	}

	readBuffer.Stop();
}

void CHDTVtoMPEG2Dlg::OnOutputfileBrowse()
{
	char path[256];

	m_OutputFile.GetWindowText(m_OutputFileName, 256);

	char *p = strrchr(m_OutputFileName, '\\');
	if (p)
		p++;
	else if (!(*m_OutputFileName))
		p = m_ExtensionStr;
	else
		p = m_OutputFileName;

	CFileDialog fdlg(
		FALSE, NULL, p, OFN_ENABLESIZING,
		"All Files|*.*||", this );

	strcpy(path, m_OutputFileName);
	if (!strrchr(path, '\\'))// if no path, use the path in the ini file
	{
		GetPrivateProfileString("Settings", "PathOut", "", path, 255, inipath);
	}
	else // use the path in the output file name
	{
		CString sPath = path;
		int index = sPath.ReverseFind( '\\' );

		sPath = sPath.Left( index );
		strcpy( path, sPath );
	}

	fdlg.m_ofn.lpstrInitialDir = path;

	if (fdlg.DoModal() == IDOK)
	{
		SaveOutputFilePath(fdlg.GetPathName());
		m_OutputFile.SetWindowText(fdlg.GetPathName());
		RemoveOutputFilenameExtension(m_ExtensionStr);
		AddOutputFilenameExtension(m_ExtensionStr);
	}
}

void CHDTVtoMPEG2Dlg::SaveOutputFilePath(const char *fileName)
{
	char buf[256];
	strcpy(buf, fileName);
	char *p = strrchr(buf, '\\');
	if (p)
	{
		p++;
		*p = '\0';
		WritePrivateProfileString("Settings", "PathOut", buf, inipath);
	}
}

// handle mouse scrolling and keyboard hot key input
BOOL CHDTVtoMPEG2Dlg::PreTranslateMessage( MSG *pMsg )
{
	CWnd *pActive;
	CWnd *pFocus;
	CRect rect;
	CPoint pt;

	// get the dialog rectangle
	GetWindowRect( rect );
	pActive = GetActiveWindow();
	pFocus = GetFocus();
	pt = pMsg->pt;


	// if the mouse is over the dialog AND the main dialog is the active window
//	if( rect.PtInRect( pt ) && pActive == this )
	if( pActive == this )
	{
		switch( pMsg->message )
		{
			case WM_MOUSEWHEEL:
				// if the input file ListBox doesn't have the focus
				if( pFocus != m_pInputFiles )
				{
					UINT nFlags;
					short zDelta;

					nFlags = GET_KEYSTATE_WPARAM(pMsg->wParam);
					zDelta = GET_WHEEL_DELTA_WPARAM(pMsg->wParam);

					OnMouseWheel( nFlags, zDelta, pt );
					return true;
				}
				break;

			case WM_CHAR:
				if( pFocus != m_pOutputFile  && pFocus != m_pExtension && pFocus != m_pMaxFileSize &&
					pFocus != m_pVideoPID    &&	pFocus != m_pAudioPID )
				{
					switch( pMsg->wParam )
					{	
						case 83:  // S
						case 115: // s
						OnSeekBack3();
						break;

						case 68:  // D
						case 100: // d
						OnSeekBack2();
						break;

						case 70:  // F
						case 102: // f
						OnSeekBack1();
						break;

						case 74:  // J
						case 106: // j
						OnSeekForward1();
						break;

						case 75:  // K
						case 107: // k
						OnSeekForward2();
						break;

						case 76:  // L
						case 108: // l
						OnSeekForward3();
						break;

						case 73:  // I
						case 105: // i
						OnBeginInclude();
						break;

						case 69:  // E
						case 101: // e
						OnBeginExclude();
						break;

						case 80:  // P
						case 112: // p
						OnPrevRegion();
						break;

						case 78:  // N
						case 110: // n
						OnNextRegion();
						break;

						case 86:  // V
						case 118: // v
						OnMoveRegionBack();
						break;

						case 77:  // M
						case 109: // m
						OnMoveRegionForward();
						break;

						case 65:  // A
						case 97:  // a
						OnAdd();
						break;

						case 79:  // O
						case 111: // o
						m_pOutputFile->SetFocus();
						break;

						case 67:  // C
						case 99:  // c
						OnConvert();
						break;

						default:
						return false;

					}// end switch( pMsg->wParam )

					return true;
				}// end if( pFocus != m_pOutputFile )

			case WM_KEYDOWN: 
				if( pFocus != m_pOutputFile  && pFocus != m_pExtension && pFocus != m_pMaxFileSize &&
					pFocus != m_pVideoPID    &&	pFocus != m_pAudioPID )
				{
					switch( pMsg->wParam )
					{
						case VK_F5: // F5 key
						RefreshInputFiles();
						return true;
					}
					
				}

			case WM_SYSCHAR:
				if( pFocus != m_pOutputFile  && pFocus != m_pExtension && pFocus != m_pMaxFileSize &&
					pFocus != m_pVideoPID    &&	pFocus != m_pAudioPID )
				{
					switch( pMsg->wParam )
					{
						case 68:  // D 
						case 100: // d
						// if the Alt key is pressed
						if(( pMsg->lParam & 0x20000000 ) == 0x20000000 )
						{
							OnRemoveRegion();
							return true;
						}
					}
				}
		}// end switch( pMsg->message )
	}// end if( rect.PtInRect( pt ) && pActive == this )

	return CDialog::PreTranslateMessage( pMsg );
}

// handle Channel check box message
void CHDTVtoMPEG2Dlg::OnChannelCheckBox()
{
	// select the appropiate pids in the PID's combo box for this channel

	int ch = m_ChannelCombo.GetCurSel();
	BOOL isChecked = m_ChannelCombo.GetCheck( ch );
	BOOL found = false;
	if( ch >= 0 && ch < m_NumChannels )
	{
		for( int i = 0; i < m_PIDCombo.GetCount(); i++ )
			// Note: videopid\audiopid are pType 1st 16 bits, pid last 16 bits
			if( m_PIDCombo.GetItemData( i ) == m_Channels[ch].videopid || m_PIDCombo.GetItemData( i ) == m_Channels[ch].audiopid )
			{
				// verify that the video pid doesn't also belong to another channel that is selected
				//  before removing its check
				if( m_PIDCombo.GetItemData( i ) == m_Channels[ch].videopid )
				{
					for( int j = 0; j < m_ChannelCombo.GetCount(); j++ )
					{
						// if the Channel is check and isn't the Channel we just changed the state on
						if( m_ChannelCombo.GetCheck( j ) && ch != j )
							// and it's video pid matches the one we picked
							if( m_Channels[j].videopid == m_Channels[ch].videopid )
								// if we are going to removed it, we can't
								if( !isChecked )
								{
									found = true;
									break;
								}
					}
					if( !found ) // OK to change the state of this box
						m_PIDCombo.SetCheck( i, isChecked ? TRUE : FALSE );
				}
				
				else // audio pid, just change its check state
					m_PIDCombo.SetCheck( i, isChecked ? TRUE : FALSE );
			}
	}

	// update the PIDDeleleList
	OnPidsCheckBox();
}

// handle PID's check box message
void CHDTVtoMPEG2Dlg::OnPidsCheckBox()
{
	int i = 0;

	// clear the list
	pPIDDeleteList->RemoveAll();
	pProgramMapPIDs->RemoveAll();
	pPCRPIDs->RemoveAll();

	// update the delete list
	for( i = 0; i < m_PIDCombo.GetCount(); ++i )
	{
		if( !m_PIDCombo.GetCheck( i ) )
			pPIDDeleteList->Add( (WORD)m_PIDCombo.GetItemData( i ) );
	}

	// add the PM pid for the channels not selected
	bool found = false;
	for( i = 0; i < m_ChannelCombo.GetCount(); i++ )
	{
		if( !m_ChannelCombo.GetCheck( i ) )
		{
			// before adding the PMT pid to the list, verify that the pid doesn't also belong
			//  to a Channel that shares the pid and that it isn't already in the list
			
			// is the pid shared with another Channel that is selected?
			for( int j = 0; j < m_ChannelCombo.GetCount(); j++ )
			{
				if( m_ChannelCombo.GetCheck( j ) )
					if( m_Channels[j].PMT_pid == m_Channels[i].PMT_pid )
					{   // can't add this pid
						found = true;
						break;
					}
			}

			if( !found )
			{
				found = false;
				// before we add it, is it already in the list?
				for( int j = 0; j < pPIDDeleteList->GetCount(); j++ )
				{
					if( pPIDDeleteList->GetAt( j ) == m_Channels[i].PMT_pid )
					{   // pid is already in the list
						found = true;
						break;
					}
				}
				if( !found ) // add the pid to the list
					// put PM pids at the top of the list
					pPIDDeleteList->InsertAt( 0, m_Channels[i].PMT_pid );
			}
		}
		else
		{
			// add this channels PM to the list of PMT pids to keep
			// is it already in the list?
			found = false;
			for( int j = 0; j < pProgramMapPIDs->GetCount(); j++ )
			{
				if( pProgramMapPIDs->GetAt( j ) == m_Channels[i].PMT_pid )
				{ // pid is already in the list
					found = true;
					break;
				}
			}

			if( !found ) // add the PM to the list
				pProgramMapPIDs->Add( m_Channels[i].PMT_pid );
		}

		found = false;
	}

	// update the PCR info
	found = false;
	for( i = 0; i < m_ChannelCombo.GetCount(); i++ )
	{
		// add the PCR pid for the channels not selected
		if( !m_ChannelCombo.GetCheck( i ) )
		{
			// before adding the PCR pid to the list, verify that the pid doesn't also belong
			//  to a Channel that shares the pid and that it isn't already in the list
			
			// is the pid shared with another Channel that is selected?
			for( int j = 0; j < m_ChannelCombo.GetCount(); j++ )
			{
				if( m_ChannelCombo.GetCheck( j ) )
					if( m_Channels[j].PCR_pid == m_Channels[i].PCR_pid )
					{   // can't add this pid
						found = true;
						break;
					}
			}

			if( !found )
			{
				found = false;
				// before we add it, is it already in the list?
				for( int j = 0; j < pPIDDeleteList->GetCount(); j++ )
				{
					if( pPIDDeleteList->GetAt( j ) == m_Channels[i].PCR_pid )
					{   // pid is already in the list
						found = true;
						break;
					}
				}
				if( !found ) // add the pid to the list
					pPIDDeleteList->Add( m_Channels[i].PCR_pid );
			}
		}
		else
		{
			// add this channels PCR to the list of PCR pids to keep
			// is it already in the list?
			found = false;
			for( int j = 0; j < pPCRPIDs->GetCount(); j++ )
			{
				if( pPCRPIDs->GetAt( j ) == m_Channels[i].PCR_pid )
				{ // pid is already in the list
					found = true;
					break;
				}
			}

			if( !found ) // add the PCR to the list
				pPCRPIDs->Add( m_Channels[i].PCR_pid );
		}

		found = false;
	}

	// update the pPidInfoList data
	Pid_Info *pPidInfo;
	for( i = 0; i < m_pPidInfoList->GetCount(); i++ )
	{
		pPidInfo = (Pid_Info*)m_pPidInfoList->GetAt( i );
		delete pPidInfo;
	}

	pPidInfo = NULL;
	m_pPidInfoList->RemoveAll();

	// add the PAT pid
//	pPidInfo = new Pid_Info();
	pPidInfo = new Pid_Info;

	pPidInfo->new_pid = -1;
	pPidInfo->original_pid = 0x00;
	pPidInfo->processed = false;
	pPidInfo->strip_pid = false;
	pPidInfo->type = PID_PAT;
	pPidInfo->continuity_counter = -1;

	m_pPidInfoList->Add( pPidInfo );

	// add the PM pids to keep
	short pid;
	for( int i = 0; i < pProgramMapPIDs->GetCount(); i++ )
	{
		pid = pProgramMapPIDs->GetAt( i );

//		pPidInfo = new Pid_Info();
		pPidInfo = new Pid_Info;

		pPidInfo->new_pid = -1;
		pPidInfo->original_pid = -1;
		pPidInfo->processed = false;
		pPidInfo->strip_pid = false;
		pPidInfo->type = PID_UNKNOWN;
		pPidInfo->continuity_counter = -1;

		pPidInfo->original_pid = pid;
		pPidInfo->type = PID_PMT;
		pPidInfo->strip_pid = false;

		m_pPidInfoList->Add( pPidInfo );
	}

	// other pids to keep
	for( i = 0; i < m_PIDCombo.GetCount(); i++ )
	{
		if( m_PIDCombo.GetCheck( i ) ) // pids to keep
		{
//			pPidInfo = new Pid_Info();
			pPidInfo = new Pid_Info;

			pPidInfo->new_pid = -1;
			pPidInfo->original_pid = -1;
			pPidInfo->processed = false;
			pPidInfo->strip_pid = false;
			pPidInfo->type = PID_UNKNOWN;
			pPidInfo->continuity_counter = -1;

			pPidInfo->original_pid = (short)m_PIDCombo.GetItemData( i );
			pPidInfo->type = (pidType)( m_PIDCombo.GetItemData( i ) >> 16 );
			pPidInfo->strip_pid = false;
			pPidInfo->processed = false;
			pPidInfo->program_number = -1;

			// tag the video pid with the program_number just in case multiple PMT
			//  reference the same video pid.  needed when updating the PSIP packets
			if( pPidInfo->type == PID_MPEG2_VIDEO || pPidInfo->type == PID_USER_PRIVATE )
			{
				for( int j = 0; j < m_ChannelCombo.GetCount(); j++ )
				{
					if(  m_ChannelCombo.GetCheck( j ))
					{
						pPidInfo->program_number = m_Channels[ j ].program_number;
						break;
					}
				}
			}
			m_pPidInfoList->Add( pPidInfo );
		}
	}

	// add the PCR pids to keep
	found = false;
	for( i = 0; i < pPCRPIDs->GetCount(); i++ )
	{
		pid = pPCRPIDs->GetAt( i );

		// if the PCR_PID == an existing PID, then the PCR_PID is the
		//  same as the Video PID so don't add it
		for( int j = 0; j < m_pPidInfoList->GetCount(); j++ )
		{
			pPidInfo = (Pid_Info*)m_pPidInfoList->GetAt( j );
			if( pid != pPidInfo->original_pid )
				continue;
			else
			{
				found = true;
				break;
			}
		}

		if( !found )
		{
//			pPidInfo = new Pid_Info();
			pPidInfo = new Pid_Info;

			pPidInfo->new_pid = -1;
			pPidInfo->original_pid = -1;
			pPidInfo->processed = false;
			pPidInfo->strip_pid = false;
			pPidInfo->type = PID_UNKNOWN;
			pPidInfo->continuity_counter = -1;

			pPidInfo->original_pid = pid;
			pPidInfo->type = PID_PCR;
			pPidInfo->strip_pid = false;

			m_pPidInfoList->Add( pPidInfo );
		}
	}

	// pids to delete
	for( i = 0; i < pPIDDeleteList->GetCount(); i++ )
	{
//		pPidInfo = new Pid_Info();
		pPidInfo = new Pid_Info;

		pPidInfo->new_pid = -1;
		pPidInfo->original_pid = -1;
		pPidInfo->processed = false;
		pPidInfo->strip_pid = false;
		pPidInfo->type = PID_UNKNOWN;
		pPidInfo->continuity_counter = -1;

		pPidInfo->original_pid = (short)pPIDDeleteList->GetAt( i );
		pPidInfo->strip_pid = true;

		m_pPidInfoList->Add( pPidInfo );
	}

	// keep track of the check state of NULL and PSIP
	for( i = 0; i < m_PIDCombo.GetCount(); i++ )
	{
		short pid = (short)m_PIDCombo.GetItemData( i );

		CString pType;
		if( pid == 0x1fff || pid == 0x1fef || pid == 0x1ffb )
		{
			if( pid == 0x1fff || pid == 0x1fef )
				WritePrivateProfileString("Settings", "KeepNULLs", m_PIDCombo.GetCheck( i ) ? "1" : "0", inipath );
			else
				WritePrivateProfileString("Settings", "KeepPSIP", m_PIDCombo.GetCheck( i ) ? "1" : "0", inipath );
		}
	}
}

void CHDTVtoMPEG2Dlg::OnBnClickedClips()
{
	m_Clips = m_Clips ? false : true;

	if( m_Clips )
		m_pMaxFileSize->EnableWindow( false );
	else
		m_pMaxFileSize->EnableWindow( true );

	// save state in ini file
	WritePrivateProfileString( "Settings", "CreateClips", m_Clips ? "1" : "0", inipath );
}

// refresh the input file(s) information without deleteing the cut info
//  used when editing a file when it's still being recorded
void CHDTVtoMPEG2Dlg::RefreshInputFiles()
{
	HANDLE f = INVALID_HANDLE_VALUE;
	TSFileInfo *pFileInfo;
	pFileInfo = m_InputList.GetHead();
	bool bCloseFile = false;

	while( pFileInfo )
	{
		f = pFileInfo->m_f;

		if( f == INVALID_HANDLE_VALUE )
		{
			f = CreateFile( pFileInfo->m_fileName,
							GENERIC_READ,
							FILE_SHARE_READ | FILE_SHARE_WRITE,
							NULL,
							OPEN_EXISTING,
							FILE_ATTRIBUTE_NORMAL | FILE_FLAG_NO_BUFFERING | FILE_FLAG_RANDOM_ACCESS, NULL);

			bCloseFile = true;
		}

		// get the size for this file
		__int64 size = 0;

		if( f != INVALID_HANDLE_VALUE )
		{
			// get file size
			DWORD hi;
			DWORD lo = GetFileSize(f, &hi);
			size = (__int64(hi) << 32) | (__int64(lo) & 0xffffffff);

			// update the size
			pFileInfo->m_fileSize = size;

			if( bCloseFile )
			{
				CloseHandle( f );
				bCloseFile = false;
			}

			pFileInfo = pFileInfo->m_next;
		}
	}

	m_InputList.RecomputeBytes();
	m_InputList.RecomputeTotalIncludedBytes();

	SetVisibleStates();
	ConfigureRegionsUI();
	UpdateRegionsUI();
	EmptyScanBuffer();
}

void CHDTVtoMPEG2Dlg::GetConfigSettings()
{
	// Preserve Bit Rate
	GetPrivateProfileString( "Settings", "PreserveBitRate", "0", temp, 2, inipath );
	if( temp[0] == '0' )
		m_PreserveBitRate = false;
	else
		m_PreserveBitRate = true;

	// change vertical_size_value
	GetPrivateProfileString( "Settings", "ChangeVerticalSize", "0", temp, 2, inipath );
	if( temp[0] == '0' )
		m_SetVerticalSizeTo1080 = false;
	else
		m_SetVerticalSizeTo1080 = true;

	// Create TRI files
	GetPrivateProfileString( "Settings", "CreateTRIFiles", "1", temp, 2, inipath );
	if( temp[0] == '0' )
		m_CreateTRIFiles = false;
	else
		m_CreateTRIFiles = true;

	// tri directory location
	GetPrivateProfileString( "Settings", "TRIOut", " ", m_TRIDirectory, 255, inipath );

	// Include black frames
	GetPrivateProfileString( "Settings", "IncludeBlackFrames", "1", temp, 2, inipath );
	if( temp[0] == '0' )
		m_IncludeBlackFrames = false;
	else
		m_IncludeBlackFrames = true;

	// max seek size
	GetPrivateProfileString( "Settings", "MaxSeekSize", "60", temp, 3, inipath );
	m_MaxSeekSize = atoi( temp );

	// Clips status
	GetPrivateProfileString( "Settings", "CreateClips", "0", temp, 2, inipath );
	if( temp[0] == '0' )
	{
		m_Clips = false;
		m_pMaxFileSize->EnableWindow( true );
	}
	else
	{
		m_Clips = true;
		m_pMaxFileSize->EnableWindow( false );
	}

	// save state in ini file
	WritePrivateProfileString( "Settings", "CreateClips", m_Clips ? "1" : "0", inipath );
}
